@@ -260,9 +260,7 @@ Magick::Color::operator std::string() const
     return std::string("none");
 
   pixel.colorspace=(_pixelType == RGBPixel || _pixelType == RGBAPixel) ?
-    RGBColorspace : CMYKColorspace;
-  pixel.alpha_trait=(_pixelType == RGBAPixel || _pixelType == CMYKAPixel) ?
-    BlendPixelTrait : UndefinedPixelTrait;
+    sRGBColorspace : CMYKColorspace;
   pixel.depth=MAGICKCORE_QUANTUM_DEPTH;
   pixel.alpha=_pixel->alpha;
   pixel.alpha_trait=_pixel->alpha_trait;
@@ -300,7 +298,7 @@ Magick::Color::PixelType Magick::Color::pixelType() const
 
 void Magick::Color::isValid(bool valid_)
 {
-  if ((valid_ && isValid()) || (!valid_ && !isValid()))
+  if (bool(valid_) == bool(isValid()))
     return;
 
   if (!_pixelOwn)
@@ -405,9 +403,9 @@ Magick::Quantum Magick::Color::scaleDoubleToQuantum(const double double_)
 double Magick::Color::scaleQuantumToDouble(const Magick::Quantum quantum_)
 {
 #if (MAGICKCORE_QUANTUM_DEPTH < 32) && (MAGICKCORE_SIZEOF_FLOAT_T != MAGICKCORE_SIZEOF_DOUBLE || !defined(MAGICKCORE_HDRI_SUPPORT))
-  return(static_cast<double>(quantum_)/QuantumRange);
+  return(static_cast<double>(QuantumScale*quantum_));
 #else
-  return(quantum_/QuantumRange);
+  return(QuantumScale*quantum_);
 #endif
 }
 
@@ -421,7 +419,7 @@ void Magick::Color::initPixel()
 void Magick::Color::setAlpha(const Magick::Quantum alpha_)
 {
   _pixel->alpha=alpha_;
-  if (alpha_ == QuantumRange)
+  if (alpha_ == OpaqueAlpha)
     {
       _pixel->alpha_trait=UndefinedPixelTrait;
       if (_pixelType == RGBAPixel)
@@ -546,7 +544,7 @@ Magick::ColorCMYK::ColorCMYK(PixelInfo *rep_,PixelType pixelType_)
 }
 
 Magick::ColorGray::ColorGray(void)
-  : Color()
+  : Color(RGBPixel)
 {
 }
 
@@ -590,7 +588,7 @@ Magick::ColorGray::ColorGray(PixelInfo *rep_,PixelType pixelType_)
 }
 
 Magick::ColorHSL::ColorHSL(void)
-  : Color()
+  : Color(RGBPixel)
 {
 }
 
@@ -601,7 +599,7 @@ Magick::ColorHSL::ColorHSL(const Magick::Color &color_)
 
 Magick::ColorHSL::ColorHSL(const double hue_,const double saturation_,
   const double lightness_)
-  : Color()
+  : Color(RGBPixel)
 {
   double
     blue,
@@ -737,7 +735,7 @@ double Magick::ColorHSL::saturation(void) const
 }
 
 Magick::ColorMono::ColorMono(void)
-  : Color()
+  : Color(RGBPixel)
 {
 }
 
@@ -780,7 +778,7 @@ Magick::ColorMono::ColorMono(PixelInfo *rep_,PixelType pixelType_)
 }
 
 Magick::ColorRGB::ColorRGB(void)
-  : Color()
+  : Color(RGBPixel)
 {
 }
 
@@ -859,7 +857,7 @@ Magick::ColorRGB::ColorRGB(PixelInfo *rep_,PixelType pixelType_)
 }
 
 Magick::ColorYUV::ColorYUV(void)
-  : Color()
+  : Color(RGBPixel)
 {
 }
 
@@ -869,7 +867,7 @@ Magick::ColorYUV::ColorYUV(const Magick::Color &color_)
 }
 
 Magick::ColorYUV::ColorYUV(const double y_,const double u_,const double v_)
-  : Color()
+  : Color(RGBPixel)
 {
   convert(y_, u_, v_);
 }
