@@ -123,10 +123,12 @@ namespace MagickCore
 #          pragma comment(lib, "CORE_DB_jp2_.lib")
 #        endif
 #        if defined(MAGICKCORE_JPEG_DELEGATE)
-#          pragma comment(lib, "CORE_DB_jpeg_.lib")
+#          pragma comment(lib, "CORE_DB_jpeg-turbo_.lib")
 #        endif
 #        if defined(MAGICKCORE_JXL_DELEGATE)
-#          pragma comment(lib, "CORE_DB_jxl_.lib")
+#          pragma comment(lib, "CORE_DB_highway_.lib")
+#          pragma comment(lib, "CORE_DB_brotli_.lib")
+#          pragma comment(lib, "CORE_DB_jpeg-xl_.lib")
 #        endif
 #        if defined(MAGICKCORE_LCMS_DELEGATE)
 #          pragma comment(lib, "CORE_DB_lcms_.lib")
@@ -149,6 +151,9 @@ namespace MagickCore
 #        if defined(MAGICKCORE_XML_DELEGATE)
 #          pragma comment(lib, "CORE_DB_libxml_.lib")
 #        endif
+#        if defined(MAGICKCORE_ZIP_DELEGATE)
+#          pragma comment(lib, "CORE_DB_libzip_.lib")
+#        endif
 #        if defined(MAGICKCORE_LQR_DELEGATE)
 #          pragma comment(lib, "CORE_DB_ffi_.lib")
 #          pragma comment(lib, "CORE_DB_glib_.lib")
@@ -159,12 +164,17 @@ namespace MagickCore
 #          pragma comment(lib, "CORE_DB_openjpeg_.lib")
 #        endif
 #        if defined(MAGICKCORE_PANGOCAIRO_DELEGATE)
+#          pragma comment(lib, "CORE_DB_fribidi_.lib")
+#          pragma comment(lib, "CORE_DB_harfbuzz_.lib")
 #          pragma comment(lib, "CORE_DB_pango_.lib")
 #          pragma comment(lib, "CORE_DB_pixman_.lib")
 #        endif
 #        if defined(MAGICKCORE_PNG_DELEGATE)
 #          pragma comment(lib, "CORE_DB_png_.lib")
 #        endif
+#        if defined(MAGICKCORE_RAQM_DELEGATE)
+#          pragma comment(lib, "CORE_DB_raqm_.lib")
+#        endif
 #        if defined(MAGICKCORE_TIFF_DELEGATE)
 #          pragma comment(lib, "CORE_DB_tiff_.lib")
 #        endif
@@ -202,10 +212,12 @@ namespace MagickCore
 #          pragma comment(lib, "CORE_RL_jp2_.lib")
 #        endif
 #        if defined(MAGICKCORE_JPEG_DELEGATE)
-#          pragma comment(lib, "CORE_RL_jpeg_.lib")
+#          pragma comment(lib, "CORE_RL_jpeg-turbo_.lib")
 #        endif
 #        if defined(MAGICKCORE_JXL_DELEGATE)
-#          pragma comment(lib, "CORE_RL_jxl_.lib")
+#          pragma comment(lib, "CORE_RL_highway_.lib")
+#          pragma comment(lib, "CORE_RL_brotli_.lib")
+#          pragma comment(lib, "CORE_RL_jpeg-xl_.lib")
 #        endif
 #        if defined(MAGICKCORE_LCMS_DELEGATE)
 #          pragma comment(lib, "CORE_RL_lcms_.lib")
@@ -228,6 +240,9 @@ namespace MagickCore
 #        if defined(MAGICKCORE_XML_DELEGATE)
 #          pragma comment(lib, "CORE_RL_libxml_.lib")
 #        endif
+#        if defined(MAGICKCORE_ZIP_DELEGATE)
+#          pragma comment(lib, "CORE_RL_libzip_.lib")
+#        endif
 #        if defined(MAGICKCORE_LQR_DELEGATE)
 #          pragma comment(lib, "CORE_RL_ffi_.lib")
 #          pragma comment(lib, "CORE_RL_glib_.lib")
@@ -238,12 +253,17 @@ namespace MagickCore
 #          pragma comment(lib, "CORE_RL_openjpeg_.lib")
 #        endif
 #        if defined(MAGICKCORE_PANGOCAIRO_DELEGATE)
+#          pragma comment(lib, "CORE_RL_fribidi_.lib")
+#          pragma comment(lib, "CORE_RL_harfbuzz_.lib")
 #          pragma comment(lib, "CORE_RL_pango_.lib")
 #          pragma comment(lib, "CORE_RL_pixman_.lib")
 #        endif
 #        if defined(MAGICKCORE_PNG_DELEGATE)
 #          pragma comment(lib, "CORE_RL_png_.lib")
 #        endif
+#        if defined(MAGICKCORE_RAQM_DELEGATE)
+#          pragma comment(lib, "CORE_RL_raqm_.lib")
+#        endif
 #        if defined(MAGICKCORE_TIFF_DELEGATE)
 #          pragma comment(lib, "CORE_RL_tiff_.lib")
 #        endif
@@ -522,6 +542,7 @@ namespace Magick
   using MagickCore::DXT1Compression;
   using MagickCore::DXT3Compression;
   using MagickCore::DXT5Compression;
+  using MagickCore::BC7Compression;
   using MagickCore::FaxCompression;
   using MagickCore::Group4Compression;
   using MagickCore::JBIG1Compression;
