@@ -17,7 +17,7 @@
 %                                October 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -157,7 +157,7 @@ static double GetEdgeBackgroundCensus(const Image *image,
   RectangleInfo
     edge_geometry;
 
-  register const Quantum
+  const Quantum
     *p;
 
   ssize_t
@@ -216,7 +216,7 @@ static double GetEdgeBackgroundCensus(const Image *image,
   edge_view=AcquireVirtualCacheView(edge_image,exception);
   for (y=0; y < (ssize_t) edge_image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
     p=GetCacheViewVirtualPixels(edge_view,0,y,edge_image->columns,1,exception);
@@ -399,13 +399,13 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     status;
 
   PixelInfo
-    target[3],
+    target[4],
     zero;
 
   RectangleInfo
     bounds;
 
-  register const Quantum
+  const Quantum
     *p;
 
   ssize_t
@@ -418,10 +418,40 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   artifact=GetImageArtifact(image,"trim:percent-background");
   if (artifact != (const char *) NULL)
     return(GetEdgeBoundingBox(image,exception));
-  bounds.width=0;
-  bounds.height=0;
-  bounds.x=(ssize_t) image->columns;
-  bounds.y=(ssize_t) image->rows;
+  artifact=GetImageArtifact(image, "trim:edges");
+  if (artifact == (const char *) NULL)
+    {
+      bounds.width=image->columns == 1 ? 1 : 0;
+      bounds.height=image->rows == 1 ? 1 : 0;
+      bounds.x=(ssize_t) image->columns;
+      bounds.y=(ssize_t) image->rows;
+    }
+  else
+    {
+      char
+        *edges,
+        *q,
+        *r;
+
+      bounds.width=(size_t) image->columns;
+      bounds.height=(size_t) image->rows;
+      bounds.x=0;
+      bounds.y=0;
+      edges=AcquireString(artifact);
+      r=edges;
+      while ((q=StringToken(",",&r)) != (char *) NULL)
+      {
+        if (LocaleCompare(q,"north") == 0)
+          bounds.y=(ssize_t) image->rows;
+        if (LocaleCompare(q,"east") == 0)
+          bounds.width=0;
+        if (LocaleCompare(q,"south") == 0)
+          bounds.height=0;
+        if (LocaleCompare(q,"west") == 0)
+          bounds.x=(ssize_t) image->columns;
+      }
+      edges=DestroyString(edges);
+    }
   GetPixelInfo(image,&target[0]);
   image_view=AcquireVirtualCacheView(image,exception);
   p=GetCacheViewVirtualPixels(image_view,0,0,1,1,exception);
@@ -441,6 +471,10 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     exception);
   if (p != (const Quantum *) NULL)
     GetPixelInfoPixel(image,p,&target[2]);
+  p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,(ssize_t)
+    image->rows-1,1,1,exception);
+  if (p != (const Quantum *) NULL)
+    GetPixelInfoPixel(image,p,&target[3]);
   status=MagickTrue;
   GetPixelInfo(image,&zero);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -455,10 +489,10 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     RectangleInfo
       bounding_box;
 
-    register const Quantum
-      *magick_restrict p;
+    const Quantum
+      *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -467,8 +501,8 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 #  pragma omp critical (MagickCore_GetImageBoundingBox)
 #endif
     bounding_box=bounds;
-    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    if (p == (const Quantum *) NULL)
+    q=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    if (q == (const Quantum *) NULL)
       {
         status=MagickFalse;
         continue;
@@ -476,7 +510,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     pixel=zero;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      GetPixelInfoPixel(image,p,&pixel);
+      GetPixelInfoPixel(image,q,&pixel);
       if ((x < bounding_box.x) &&
           (IsFuzzyEquivalencePixelInfo(&pixel,&target[0]) == MagickFalse))
         bounding_box.x=x;
@@ -489,7 +523,14 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
       if ((y > (ssize_t) bounding_box.height) &&
           (IsFuzzyEquivalencePixelInfo(&pixel,&target[2]) == MagickFalse))
         bounding_box.height=(size_t) y;
-      p+=GetPixelChannels(image);
+      if ((x < (ssize_t) bounding_box.width) &&
+          (y > (ssize_t) bounding_box.height) &&
+          (IsFuzzyEquivalencePixelInfo(&pixel,&target[3]) == MagickFalse))
+        {
+          bounding_box.width=(size_t) x;
+          bounding_box.height=(size_t) y;
+        }
+      q+=GetPixelChannels(image);
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
 #  pragma omp critical (MagickCore_GetImageBoundingBox)
@@ -567,12 +608,13 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
     background[4],
     edge_background;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
     Most dominant color of edges/corners is the background color of the image.
   */
+  memset(&edge_background,0,sizeof(edge_background));
   artifact=GetImageArtifact(image,"convex-hull:background-color");
   if (artifact == (const char *) NULL)
     artifact=GetImageArtifact(image,"background");
@@ -596,7 +638,7 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
     RectangleInfo
       edge_geometry;
 
-    register const Quantum
+    const Quantum
       *p;
 
     ssize_t
@@ -614,6 +656,7 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
         gravity=WestGravity;
         edge_geometry.width=1;
         edge_geometry.height=0;
+        break;
       }
       case 1:
       {
@@ -622,6 +665,7 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
         gravity=EastGravity;
         edge_geometry.width=1;
         edge_geometry.height=0;
+        break;
       }
       case 2:
       {
@@ -629,6 +673,7 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
         gravity=NorthGravity;
         edge_geometry.width=0;
         edge_geometry.height=1;
+        break;
       }
       case 3:
       {
@@ -637,6 +682,7 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
         gravity=SouthGravity;
         edge_geometry.width=0;
         edge_geometry.height=1;
+        break;
       }
     }
     GetPixelInfoPixel(image,p,background+i);
@@ -650,7 +696,7 @@ static PixelInfo GetEdgeBackgroundColor(const Image *image,
     edge_view=AcquireVirtualCacheView(edge_image,exception);
     for (y=0; y < (ssize_t) edge_image->rows; y++)
     {
-      register ssize_t
+      ssize_t
         x;
 
       p=GetCacheViewVirtualPixels(edge_view,0,y,edge_image->columns,1,
@@ -684,7 +730,7 @@ void TraceConvexHull(PointInfo *vertices,size_t number_vertices,
   PointInfo
     **chain;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -724,6 +770,7 @@ MagickExport PointInfo *GetImageConvexHull(const Image *image,
     status;
 
   MemoryInfo
+    *monotone_info,
     *vertices_info;
 
   PixelInfo
@@ -750,28 +797,29 @@ MagickExport PointInfo *GetImageConvexHull(const Image *image,
   *number_vertices=0;
   vertices_info=AcquireVirtualMemory(image->columns,image->rows*
     sizeof(*vertices));
-  monotone_chain=(PointInfo **) AcquireQuantumMemory(2*image->columns,2*
+  monotone_info=AcquireVirtualMemory(2*image->columns,2*
     image->rows*sizeof(*monotone_chain));
   if ((vertices_info == (MemoryInfo *) NULL) ||
-      (monotone_chain == (PointInfo **) NULL))
+      (monotone_info == (MemoryInfo *) NULL))
     {
-      if (monotone_chain != (PointInfo **) NULL)
-        monotone_chain=(PointInfo **) RelinquishMagickMemory(monotone_chain);
+      if (monotone_info != (MemoryInfo *) NULL)
+        monotone_info=(MemoryInfo *) RelinquishVirtualMemory(monotone_info);
       if (vertices_info != (MemoryInfo *) NULL)
         vertices_info=RelinquishVirtualMemory(vertices_info);
       return((PointInfo *) NULL);
     }
   vertices=(PointInfo *) GetVirtualMemoryBlob(vertices_info);
+  monotone_chain=(PointInfo **) GetVirtualMemoryBlob(monotone_info);
   image_view=AcquireVirtualCacheView(image,exception);
   background=GetEdgeBackgroundColor(image,image_view,exception);
   status=MagickTrue;
   n=0;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const Quantum
+    const Quantum
       *p;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -807,7 +855,7 @@ MagickExport PointInfo *GetImageConvexHull(const Image *image,
   if (convex_hull != (PointInfo *) NULL)
     for (n=0; n < *number_vertices; n++)
       convex_hull[n]=(*monotone_chain[n]);
-  monotone_chain=(PointInfo **) RelinquishMagickMemory(monotone_chain);
+  monotone_info=RelinquishVirtualMemory(monotone_info);
   vertices_info=RelinquishVirtualMemory(vertices_info);
   return(convex_hull);
 }
@@ -844,7 +892,7 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -913,7 +961,9 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
     }
   image_view=AcquireVirtualCacheView(image,exception);
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
+  DisableMSCWarning(4127)
   if ((1UL*QuantumRange) <= MaxMap)
+  RestoreMSCWarning
     {
       size_t
         *depth_map;
@@ -926,10 +976,7 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
         ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
       for (i=0; i <= (ssize_t) MaxMap; i++)
       {
-        unsigned int
-          depth;
-
-        for (depth=1; depth < MAGICKCORE_QUANTUM_DEPTH; depth++)
+        for (depth=1; depth < (size_t) MAGICKCORE_QUANTUM_DEPTH; depth++)
         {
           Quantum
             pixel;
@@ -953,10 +1000,10 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
         const int
           id = GetOpenMPThreadId();
 
-        register const Quantum
+        const Quantum
           *magick_restrict p;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -966,17 +1013,17 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
           continue;
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          register ssize_t
-            i;
+          ssize_t
+            j;
 
-          for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
+          for (j=0; j < (ssize_t) GetPixelChannels(image); j++)
           {
-            PixelChannel channel = GetPixelChannelChannel(image,i);
+            PixelChannel channel = GetPixelChannelChannel(image,j);
             PixelTrait traits = GetPixelChannelTraits(image,channel);
             if ((traits & UpdatePixelTrait) == 0)
               continue;
-            if (depth_map[ScaleQuantumToMap(p[i])] > current_depth[id])
-              current_depth[id]=depth_map[ScaleQuantumToMap(p[i])];
+            if (depth_map[ScaleQuantumToMap(p[j])] > current_depth[id])
+              current_depth[id]=depth_map[ScaleQuantumToMap(p[j])];
           }
           p+=GetPixelChannels(image);
         }
@@ -1005,10 +1052,10 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
     const int
       id = GetOpenMPThreadId();
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -1018,10 +1065,10 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
       continue;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
-        i;
+      ssize_t
+        j;
 
-      for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
+      for (j=0; j < (ssize_t) GetPixelChannels(image); j++)
       {
         PixelChannel
           channel;
@@ -1029,7 +1076,7 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
         PixelTrait
           traits;
 
-        channel=GetPixelChannelChannel(image,i);
+        channel=GetPixelChannelChannel(image,j);
         traits=GetPixelChannelTraits(image,channel);
         if ((traits & UpdatePixelTrait) == 0)
           continue;
@@ -1039,7 +1086,7 @@ MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
             range;
 
           range=GetQuantumRange(current_depth[id]);
-          if (p[i] == ScaleAnyToQuantum(ScaleQuantumToAny(p[i],range),range))
+          if (p[j] == ScaleAnyToQuantum(ScaleQuantumToAny(p[j],range),range))
             break;
           current_depth[id]++;
         }
@@ -1167,7 +1214,7 @@ MagickExport PointInfo *GetImageMinimumBoundingBox(Image *image,
     *bounding_box,
     *vertices;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1207,7 +1254,7 @@ MagickExport PointInfo *GetImageMinimumBoundingBox(Image *image,
       min_diameter = -1.0,
       min_projection = 0.0;
 
-    register ssize_t
+    ssize_t
       j,
       k;
 
@@ -1218,9 +1265,6 @@ MagickExport PointInfo *GetImageMinimumBoundingBox(Image *image,
 
     for (j=0; j < (ssize_t) number_hull_vertices; j++)
     {
-      double
-        diameter;
-
       diameter=fabs(getFeretDiameter(&vertices[i],
         &vertices[(i+1) % number_hull_vertices],&vertices[j]));
       if (min_diameter < diameter)
@@ -1523,10 +1567,10 @@ MagickExport ImageType IdentifyImageGray(const Image *image,
   ImageType
     type;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -1605,10 +1649,10 @@ MagickExport MagickBooleanType IdentifyImageMonochrome(const Image *image,
   MagickBooleanType
     bilevel;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register const Quantum
+  const Quantum
     *p;
 
   ssize_t
@@ -1680,6 +1724,9 @@ MagickExport MagickBooleanType IdentifyImageMonochrome(const Image *image,
 MagickExport ImageType IdentifyImageType(const Image *image,
   ExceptionInfo *exception)
 {
+  ImageType
+    type;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
@@ -1690,14 +1737,10 @@ MagickExport ImageType IdentifyImageType(const Image *image,
         return(ColorSeparationType);
       return(ColorSeparationAlphaType);
     }
-  if (IdentifyImageMonochrome(image,exception) != MagickFalse)
-    return(BilevelType);
-  if (IdentifyImageGray(image,exception) != UndefinedType)
-    {
-      if (image->alpha_trait != UndefinedPixelTrait)
-        return(GrayscaleAlphaType);
-      return(GrayscaleType);
-    }
+  type=IdentifyImageGray(image,exception);
+  if ((type == BilevelType) || (type == GrayscaleType) ||
+      (type == GrayscaleAlphaType))
+    return(type);
   if (IdentifyPaletteImage(image,exception) != MagickFalse)
     {
       if (image->alpha_trait != UndefinedPixelTrait)
@@ -1807,10 +1850,10 @@ MagickExport MagickBooleanType IsImageOpaque(const Image *image,
   CacheView
     *image_view;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -1900,7 +1943,7 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
   range=GetQuantumRange(depth);
   if (image->storage_class == PseudoClass)
     {
-      register ssize_t
+      ssize_t
         i;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -1926,12 +1969,14 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
   status=MagickTrue;
   image_view=AcquireAuthenticCacheView(image,exception);
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
+  DisableMSCWarning(4127)
   if ((1UL*QuantumRange) <= MaxMap)
+  RestoreMSCWarning
     {
       Quantum
         *depth_map;
 
-      register ssize_t
+      ssize_t
         i;
 
       /*
@@ -1949,10 +1994,10 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register ssize_t
+        ssize_t
           x;
 
-        register Quantum
+        Quantum
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -1966,10 +2011,10 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
           }
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          register ssize_t
-            i;
+          ssize_t
+            j;
 
-          for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
+          for (j=0; j < (ssize_t) GetPixelChannels(image); j++)
           {
             PixelChannel
               channel;
@@ -1977,11 +2022,11 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
             PixelTrait
               traits;
 
-            channel=GetPixelChannelChannel(image,i);
+            channel=GetPixelChannelChannel(image,j);
             traits=GetPixelChannelTraits(image,channel);
             if ((traits & UpdatePixelTrait) == 0)
               continue;
-            q[i]=depth_map[ScaleQuantumToMap(q[i])];
+            q[j]=depth_map[ScaleQuantumToMap(q[j])];
           }
           q+=GetPixelChannels(image);
         }
@@ -2007,10 +2052,10 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2023,7 +2068,7 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
+      ssize_t
         i;
 
       for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
