@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -333,13 +333,13 @@ static MagickBooleanType CompositeOverImage(Image *image,
       canvas_pixel,
       source_pixel;
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -367,7 +367,7 @@ static MagickBooleanType CompositeOverImage(Image *image,
           }
         pixels=p;
         if (x_offset < 0)
-          p-=x_offset*GetPixelChannels(source_image);
+          p-=x_offset*(ssize_t) GetPixelChannels(source_image);
       }
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (Quantum *) NULL)
@@ -391,7 +391,7 @@ static MagickBooleanType CompositeOverImage(Image *image,
         Sc,
         Sca;
 
-      register ssize_t
+      ssize_t
         i;
 
       size_t
@@ -512,10 +512,10 @@ static MagickBooleanType CompositeOverImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_CompositeImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,CompositeImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,CompositeImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -579,8 +579,6 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
   source_image=CloneImage(composite,0,0,MagickTrue,exception);
   if (source_image == (const Image *) NULL)
     return(MagickFalse);
-  if (IsGrayColorspace(image->colorspace) != MagickFalse)
-    (void) SetImageColorspace(image,sRGBColorspace,exception);
   (void) SetImageColorspace(source_image,image->colorspace,exception);
   if ((compose == OverCompositeOp) || (compose == SrcOverCompositeOp))
     {
@@ -611,6 +609,9 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         break;
       if ((y_offset+(ssize_t) source_image->rows) > (ssize_t) image->rows)
         break;
+      if ((source_image->alpha_trait == UndefinedPixelTrait) &&
+          (image->alpha_trait != UndefinedPixelTrait))
+        (void) SetImageAlphaChannel(source_image,OpaqueAlphaChannel,exception);
       status=MagickTrue;
       source_view=AcquireVirtualCacheView(source_image,exception);
       image_view=AcquireAuthenticCacheView(image,exception);
@@ -623,13 +624,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *p;
 
-        register Quantum
+        Quantum
           *q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -645,7 +646,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           }
         for (x=0; x < (ssize_t) source_image->columns; x++)
         {
-          register ssize_t
+          ssize_t
             i;
 
           if (GetPixelReadMask(source_image,p) <= (QuantumRange/2))
@@ -654,18 +655,16 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               q+=GetPixelChannels(image);
               continue;
             }
-          for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
+          for (i=0; i < (ssize_t) GetPixelChannels(source_image); i++)
           {
-            PixelChannel channel = GetPixelChannelChannel(image,i);
-            PixelTrait traits = GetPixelChannelTraits(image,channel);
-            PixelTrait source_traits=GetPixelChannelTraits(source_image,
+            PixelChannel channel = GetPixelChannelChannel(source_image,i);
+            PixelTrait source_traits = GetPixelChannelTraits(source_image,
               channel);
-            if (traits == UndefinedPixelTrait)
+            PixelTrait traits = GetPixelChannelTraits(image,channel);
+            if ((source_traits == UndefinedPixelTrait) ||
+                (traits == UndefinedPixelTrait))
               continue;
-            if (source_traits != UndefinedPixelTrait)
-              SetPixelChannel(image,channel,p[i],q);
-            else if (channel == AlphaPixelChannel)
-              SetPixelChannel(image,channel,OpaqueAlpha,q);
+            SetPixelChannel(image,channel,p[i],q);
           }
           p+=GetPixelChannels(source_image);
           q+=GetPixelChannels(image);
@@ -678,11 +677,8 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical (MagickCore_CompositeImage)
-#endif
-            proceed=SetImageProgress(image,CompositeImageTag,
-              (MagickOffsetType) y,image->rows);
+            proceed=SetImageProgress(image,CompositeImageTag,(MagickOffsetType)
+              y,image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
@@ -712,13 +708,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *p;
 
-        register Quantum
+        Quantum
           *q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -754,11 +750,8 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical (MagickCore_CompositeImage)
-#endif
-            proceed=SetImageProgress(image,CompositeImageTag,
-              (MagickOffsetType) y,image->rows);
+            proceed=SetImageProgress(image,CompositeImageTag,(MagickOffsetType)
+              y,image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
@@ -784,7 +777,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       CacheView
         *canvas_view;
 
-      MagickRealType
+      double
         angle_range,
         angle_start,
         height,
@@ -800,13 +793,11 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         blur;
 
       /*
-        Blur Image by resampling.
-
-        Blur Image dictated by an overlay gradient map: X = red_channel;
-          Y = green_channel; compose:args =  x_scale[,y_scale[,angle]].
+        Blur Image by resampling dictated by an overlay gradient map:
+          X = red_channel; Y = green_channel; compose:args =
+          x_scale[,y_scale[,angle]].
       */
-      canvas_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-        exception);
+      canvas_image=CloneImage(image,0,0,MagickTrue,exception);
       if (canvas_image == (Image *) NULL)
         {
           source_image=DestroyImage(source_image);
@@ -829,12 +820,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         }
       /*
         Users input sigma now needs to be converted to the EWA ellipse size.
-        The filter defaults to a sigma of 0.5 so to make this match the
-        users input the ellipse size needs to be doubled.
+        The filter defaults to a sigma of 0.5 so to make this match the users
+        input the ellipse size needs to be doubled.
       */
-      width=height=geometry_info.rho*2.0;
-      if ((flags & HeightValue) != 0 )
-        height=geometry_info.sigma*2.0;
+      width=2.0*geometry_info.rho;
+      height=width;
+      if ((flags & HeightValue) != 0)
+        height=2.0*geometry_info.sigma;
       /*
         Default the unrotated ellipse width and height axis vectors.
       */
@@ -842,23 +834,27 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       blur.x2=0.0;
       blur.y1=0.0;
       blur.y2=height;
-      /* rotate vectors if a rotation angle is given */
       if ((flags & XValue) != 0 )
         {
           MagickRealType
             angle;
 
+          /*
+            Rotate vectors if a rotation angle is given.
+          */
           angle=DegreesToRadians(geometry_info.xi);
           blur.x1=width*cos(angle);
           blur.x2=width*sin(angle);
           blur.y1=(-height*sin(angle));
           blur.y2=height*cos(angle);
         }
-      /* Otherwise lets set a angle range and calculate in the loop */
       angle_start=0.0;
       angle_range=0.0;
       if ((flags & YValue) != 0 )
         {
+          /*
+            Lets set a angle range and calculate in the loop.
+          */
           angle_start=DegreesToRadians(geometry_info.xi);
           angle_range=DegreesToRadians(geometry_info.psi)-angle_start;
         }
@@ -867,16 +863,15 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
 
         As the minimum ellipse radius of support*1.0 the EWA algorithm
         can only produce a minimum blur of 0.5 for Gaussian (support=2.0)
-        This means that even 'No Blur' will be still a little blurry!
-
-        The solution (as well as the problem of preventing any user
-        expert filter settings, is to set our own user settings, then
-        restore them afterwards.
+        This means that even 'No Blur' will be still a little blurry! The
+        solution (as well as the problem of preventing any user expert filter
+        settings, is to set our own user settings, restore them afterwards.
       */
       resample_filter=AcquireResampleFilter(image,exception);
       SetResampleFilter(resample_filter,GaussianFilter);
-
-      /* do the variable blurring of each pixel in image */
+      /*
+        Perform the variable blurring of each pixel in image.
+      */
       GetPixelInfo(image,&pixel);
       source_view=AcquireVirtualCacheView(source_image,exception);
       canvas_view=AcquireAuthenticCacheView(canvas_image,exception);
@@ -885,13 +880,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *magick_restrict p;
 
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (((y+y_offset) < 0) || ((y+y_offset) >= (ssize_t) image->rows))
@@ -909,7 +904,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               p+=GetPixelChannels(source_image);
               continue;
             }
-          if (fabs((double) angle_range) > MagickEpsilon)
+          if (fabs(angle_range) > MagickEpsilon)
             {
               MagickRealType
                 angle;
@@ -921,13 +916,6 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               blur.y1=(-height*sin(angle));
               blur.y2=height*cos(angle);
             }
-#if 0
-          if ( x == 10 && y == 60 ) {
-            (void) fprintf(stderr, "blur.x=%lf,%lf, blur.y=%lf,%lf\n",blur.x1,
-              blur.x2,blur.y1, blur.y2);
-            (void) fprintf(stderr, "scaled by=%lf,%lf\n",QuantumScale*
-              GetPixelRed(p),QuantumScale*GetPixelGreen(p));
-#endif
           ScaleResampleFilter(resample_filter,
             blur.x1*QuantumScale*GetPixelRed(source_image,p),
             blur.y1*QuantumScale*GetPixelGreen(source_image,p),
@@ -972,8 +960,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           X = red_channel;  Y = green_channel;
           compose:args = x_scale[,y_scale[,center.x,center.y]]
       */
-      canvas_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-        exception);
+      canvas_image=CloneImage(image,0,0,MagickTrue,exception);
       if (canvas_image == (Image *) NULL)
         {
           source_image=DestroyImage(source_image);
@@ -1064,13 +1051,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *magick_restrict p;
 
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (((y+y_offset) < 0) || ((y+y_offset) >= (ssize_t) image->rows))
@@ -1177,7 +1164,12 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       SetGeometryInfo(&geometry_info);
       value=GetImageArtifact(image,"compose:args");
       if (value != (char *) NULL)
-        (void) ParseGeometry(value,&geometry_info);
+        {
+          flags=ParseGeometry(value,&geometry_info);
+          if (flags == NoValue)
+            (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+              "InvalidGeometry","`%s'",value);
+        }
       break;
     }
     case ModulateCompositeOp:
@@ -1244,13 +1236,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       canvas_pixel,
       source_pixel;
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -1278,7 +1270,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           }
         pixels=p;
         if (x_offset < 0)
-          p-=x_offset*GetPixelChannels(source_image);
+          p-=x_offset*(ssize_t) GetPixelChannels(source_image);
       }
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (Quantum *) NULL)
@@ -1307,7 +1299,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         Sc,
         Sca;
 
-      register ssize_t
+      ssize_t
         i;
 
       size_t
@@ -1419,23 +1411,28 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         case DivideDstCompositeOp:
         case DivideSrcCompositeOp:
         case ExclusionCompositeOp:
+        case FreezeCompositeOp:
         case HardLightCompositeOp:
         case HardMixCompositeOp:
+        case InterpolateCompositeOp:
+        case LightenCompositeOp:
         case LinearBurnCompositeOp:
         case LinearDodgeCompositeOp:
         case LinearLightCompositeOp:
-        case LightenCompositeOp:
         case MathematicsCompositeOp:
         case MinusDstCompositeOp:
         case MinusSrcCompositeOp:
-        case ModulusAddCompositeOp:
-        case ModulusSubtractCompositeOp:
         case MultiplyCompositeOp:
+        case NegateCompositeOp:
         case OverlayCompositeOp:
         case PegtopLightCompositeOp:
         case PinLightCompositeOp:
+        case ReflectCompositeOp:
         case ScreenCompositeOp:
+        case SoftBurnCompositeOp:
+        case SoftDodgeCompositeOp:
         case SoftLightCompositeOp:
+        case StampCompositeOp:
         case VividLightCompositeOp:
         {
           alpha=RoundToUnity(Sa+Da-Sa*Da);
@@ -1484,6 +1481,26 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           alpha=Sa+Da-2.0*Sa*Da;
           break;
         }
+        case ModulusAddCompositeOp:
+        {
+          if ((Sa+Da) <= 1.0)
+            {
+              alpha=(Sa+Da);
+              break;
+            }
+          alpha=((Sa+Da)-1.0);
+          break;
+        }
+        case ModulusSubtractCompositeOp:
+        {
+          if ((Sa-Da) >= 0.0)
+            {
+              alpha=(Sa-Da);
+              break;
+            }
+          alpha=((Sa-Da)+1.0);
+          break;
+        }
         default:
         {
           alpha=1.0;
@@ -1496,6 +1513,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         case HueCompositeOp:
         case LuminizeCompositeOp:
         case ModulateCompositeOp:
+        case RMSECompositeOp:
         case SaturateCompositeOp:
         {
           GetPixelInfoPixel(source_image,p,&source_pixel);
@@ -1513,10 +1531,11 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
 
         PixelChannel channel = GetPixelChannelChannel(image,i);
         PixelTrait traits = GetPixelChannelTraits(image,channel);
-        PixelTrait source_traits=GetPixelChannelTraits(source_image,channel);
+        PixelTrait source_traits = GetPixelChannelTraits(source_image,channel);
         if (traits == UndefinedPixelTrait)
           continue;
-        if (channel == AlphaPixelChannel)
+        if ((channel == AlphaPixelChannel) &&
+            ((traits & UpdatePixelTrait) != 0))
           {
             /*
               Set alpha channel.
@@ -1545,19 +1564,10 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               }
               case ChangeMaskCompositeOp:
               {
-                MagickBooleanType
-                  equivalent;
-
-                if (Da < 0.5)
-                  {
-                    pixel=(MagickRealType) TransparentAlpha;
-                    break;
-                  }
-                equivalent=IsFuzzyEquivalencePixel(source_image,p,image,q);
-                if (equivalent != MagickFalse)
+                if (IsFuzzyEquivalencePixel(source_image,p,image,q) != MagickFalse)
                   pixel=(MagickRealType) TransparentAlpha;
                 else
-                  pixel=(MagickRealType) OpaqueAlpha;
+                  pixel=QuantumRange*Da;
                 break;
               }
               case ClearCompositeOp:
@@ -1568,6 +1578,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               case ColorizeCompositeOp:
               case HueCompositeOp:
               case LuminizeCompositeOp:
+              case RMSECompositeOp:
               case SaturateCompositeOp:
               {
                 if (fabs((double) (QuantumRange*Sa-TransparentAlpha)) < MagickEpsilon)
@@ -1596,6 +1607,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
                   pixel=QuantumRange*Sa;
                 break;
               }
+              case BlurCompositeOp:
               case CopyCompositeOp:
               case DisplaceCompositeOp:
               case DistortCompositeOp:
@@ -1612,6 +1624,25 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
                   Da*GetPixelIntensity(image,q) ? Sa : Da;
                 break;
               }
+              case DifferenceCompositeOp:
+              {
+                pixel=QuantumRange*fabs((double) (Sa-Da));
+                break;
+              }
+              case FreezeCompositeOp:
+              {
+                pixel=QuantumRange*(1.0-(1.0-Sa)*(1.0-Sa)*
+                  PerceptibleReciprocal(Da));
+                if (pixel < 0.0)
+                  pixel=0.0;
+                break;
+              }
+              case InterpolateCompositeOp:
+              {
+                pixel=QuantumRange*(0.5-0.25*cos(MagickPI*Sa)-0.25*
+                  cos(MagickPI*Da));
+                break;
+              }
               case LightenIntensityCompositeOp:
               {
                 pixel=Sa*GetPixelIntensity(source_image,p) >
@@ -1628,6 +1659,28 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
                 pixel=QuantumRange*Sa*Da;
                 break;
               }
+              case NegateCompositeOp:
+              {
+                pixel=QuantumRange*((1.0-Sa-Da));
+                break;
+              }
+              case ReflectCompositeOp:
+              {
+                pixel=QuantumRange*(Sa*Sa*PerceptibleReciprocal(1.0-Da));
+                if (pixel > QuantumRange)
+                  pixel=QuantumRange;
+                break;
+              }
+              case StampCompositeOp:
+              {
+                pixel=QuantumRange*(Sa+Da*Da-1.0);
+                break;
+              }
+              case StereoCompositeOp:
+              {
+                pixel=QuantumRange*(Sa+Da)/2;
+                break;
+              }
               default:
               {
                 pixel=QuantumRange*alpha;
@@ -1697,7 +1750,6 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=gamma*(source_dissolve*Sa*Sc+canvas_dissolve*Da*Dc);
             break;
           }
-          case BlurCompositeOp:
           case CopyCompositeOp:
           case ReplaceCompositeOp:
           case SrcCompositeOp:
@@ -1705,6 +1757,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=QuantumRange*Sca;
             break;
           }
+          case BlurCompositeOp:
           case DisplaceCompositeOp:
           case DistortCompositeOp:
           {
@@ -1790,8 +1843,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           case CopyBlackCompositeOp:
           {
             if (channel == BlackPixelChannel)
-              pixel=(MagickRealType) (QuantumRange-
-                GetPixelBlack(source_image,p));
+              pixel=(MagickRealType) GetPixelBlack(source_image,p);
             break;
           }
           case CopyBlueCompositeOp:
@@ -1892,12 +1944,12 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           }
           case DstInCompositeOp:
           {
-            pixel=QuantumRange*(Dca*Sa);
+            pixel=QuantumRange*gamma*(Dca*Sa);
             break;
           }
           case DstOutCompositeOp:
           {
-            pixel=QuantumRange*(Dca*(1.0-Sa));
+            pixel=QuantumRange*gamma*(Dca*(1.0-Sa));
             break;
           }
           case DstOverCompositeOp:
@@ -1911,6 +1963,14 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               Dca*(1.0-Sa));
             break;
           }
+          case FreezeCompositeOp:
+          {
+            pixel=QuantumRange*gamma*(1.0-(1.0-Sca)*(1.0-Sca)*
+              PerceptibleReciprocal(Dca));
+            if (pixel < 0.0)
+              pixel=0.0;
+            break;
+          }
           case HardLightCompositeOp:
           {
             if ((2.0*Sca) < Sa)
@@ -1960,6 +2020,12 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=QuantumRange*(Sca*Da);
             break;
           }
+          case InterpolateCompositeOp:
+          {
+            pixel=QuantumRange*(0.5-0.25*cos(MagickPI*Sca)-0.25*
+              cos(MagickPI*Dca));
+            break;
+          }
           case LinearBurnCompositeOp:
           {
             /*
@@ -2107,22 +2173,22 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           }
           case ModulusAddCompositeOp:
           {
-            pixel=Sc+Dc;
-            while (pixel > QuantumRange)
-              pixel-=QuantumRange;
-            while (pixel < 0.0)
-              pixel+=QuantumRange;
-            pixel=(Sa*Da*pixel+Sa*Sc*(1.0-Da)+Da*Dc*(1.0-Sa));
+            if ((Sca+Dca) <= 1.0)
+              {
+                pixel=QuantumRange*(Sca+Dca);
+                break;
+              }
+            pixel=QuantumRange*((Sca+Dca)-1.0);
             break;
           }
           case ModulusSubtractCompositeOp:
           {
-            pixel=Sc-Dc;
-            while (pixel > QuantumRange)
-              pixel-=QuantumRange;
-            while (pixel < 0.0)
-              pixel+=QuantumRange;
-            pixel=(Sa*Da*pixel+Sa*Sc*(1.0-Da)+Da*Dc*(1.0-Sa));
+            if ((Sca-Dca) >= 0.0)
+              {
+                pixel=QuantumRange*(Sca-Dca);
+                break;
+              }
+            pixel=QuantumRange*((Sca-Dca)+1.0);
             break;
           }
           case MultiplyCompositeOp:
@@ -2130,6 +2196,11 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=QuantumRange*gamma*(Sca*Dca+Sca*(1.0-Da)+Dca*(1.0-Sa));
             break;
           }
+          case NegateCompositeOp:
+          {
+            pixel=QuantumRange*(1.0-fabs(1.0-Sca-Dca));
+            break;
+          }
           case OutCompositeOp:
           case SrcOutCompositeOp:
           {
@@ -2166,7 +2237,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             */
             if (fabs((double) Da) < MagickEpsilon)
               {
-                pixel=QuantumRange*gamma*(Sca);
+                pixel=QuantumRange*gamma*Sca;
                 break;
               }
             pixel=QuantumRange*gamma*(Dca*Dca*(Sa-2.0*Sca)/Da+Sca*(2.0*Dca+1.0-
@@ -2199,6 +2270,44 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=QuantumRange*(Sca+Dca);
             break;
           }
+          case ReflectCompositeOp:
+          {
+            pixel=QuantumRange*gamma*(Sca*Sca*PerceptibleReciprocal(1.0-Dca));
+            if (pixel > QuantumRange)
+              pixel=QuantumRange;
+            break;
+          }
+          case RMSECompositeOp:
+          {
+            double
+              gray;
+
+            if (fabs((double) (QuantumRange*Sa-TransparentAlpha)) < MagickEpsilon)
+              {
+                pixel=Dc;
+                break;
+              }
+            if (fabs((double) (QuantumRange*Da-TransparentAlpha)) < MagickEpsilon)
+              {
+                pixel=Sc;
+                break;
+              }
+            gray=sqrt(
+              (canvas_pixel.red-source_pixel.red)*
+              (canvas_pixel.red-source_pixel.red)+
+              (canvas_pixel.green-source_pixel.green)*
+              (canvas_pixel.green-source_pixel.green)+
+              (canvas_pixel.blue-source_pixel.blue)*
+              (canvas_pixel.blue-source_pixel.blue)/3.0);
+            switch (channel)
+            {
+              case RedPixelChannel: pixel=gray; break;
+              case GreenPixelChannel: pixel=gray; break;
+              case BluePixelChannel: pixel=gray; break;
+              default: pixel=Dc; break;
+            }
+            break;
+          }
           case SaturateCompositeOp:
           {
             if (fabs((double) (QuantumRange*Sa-TransparentAlpha)) < MagickEpsilon)
@@ -2235,6 +2344,24 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=QuantumRange*gamma*(Sca+Dca-Sca*Dca);
             break;
           }
+          case SoftBurnCompositeOp:
+          {
+            if ((Sca+Dca) < 1.0)
+              pixel=QuantumRange*gamma*(0.5*Dca*PerceptibleReciprocal(1.0-Sca));
+            else
+              pixel=QuantumRange*gamma*(1.0-0.5*(1.0-Sca)*
+                PerceptibleReciprocal(Dca));
+            break;
+          }
+          case SoftDodgeCompositeOp:
+          {
+            if ((Sca+Dca) < 1.0)
+              pixel=QuantumRange*gamma*(0.5*Sca*PerceptibleReciprocal(1.0-Dca));
+            else
+              pixel=QuantumRange*gamma*(1.0-0.5*(1.0-Dca)*
+                PerceptibleReciprocal(Sca));
+            break;
+          }
           case SoftLightCompositeOp:
           {
             if ((2.0*Sca) < Sa)
@@ -2254,6 +2381,11 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               DcaDa)+Sca*(1.0-Da)+Dca*(1.0-Sa));
             break;
           }
+          case StampCompositeOp:
+          {
+            pixel=QuantumRange*(Sca+Dca*Dca-1.0);
+            break;
+          }
           case StereoCompositeOp:
           {
             if (channel == RedPixelChannel)
@@ -2325,10 +2457,10 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_CompositeImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,CompositeImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,CompositeImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -2411,7 +2543,7 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
       */
       for (y=0; y < (ssize_t) image->rows; y+=(ssize_t) texture_image->rows)
       {
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -2461,14 +2593,14 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
     MagickBooleanType
       sync;
 
-    register const Quantum
+    const Quantum
       *p,
       *pixels;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *q;
 
     size_t
@@ -2487,7 +2619,7 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
       }
     for (x=0; x < (ssize_t) image->columns; x+=(ssize_t) texture_image->columns)
     {
-      register ssize_t
+      ssize_t
         j;
 
       p=pixels;
@@ -2496,7 +2628,7 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
         width=image->columns-x;
       for (j=0; j < (ssize_t) width; j++)
       {
-        register ssize_t
+        ssize_t
           i;
 
         for (i=0; i < (ssize_t) GetPixelChannels(texture_image); i++)
