@@ -17,13 +17,13 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -463,13 +463,10 @@ MagickExport void ConvertHSLToRGB(const double hue,const double saturation,
   const double lightness,double *red,double *green,double *blue)
 {
   double
-    b,
-    g,
-    r,
-    v,
-    x,
-    y,
-    z;
+    c,
+    h,
+    min,
+    x;
 
   /*
     Convert HSL to RGB colorspace.
@@ -477,31 +474,66 @@ MagickExport void ConvertHSLToRGB(const double hue,const double saturation,
   assert(red != (double *) NULL);
   assert(green != (double *) NULL);
   assert(blue != (double *) NULL);
-  if (saturation == 0.0)
-    {
-      *red=(double) QuantumRange*lightness;
-      *green=(double) QuantumRange*lightness;
-      *blue=(double) QuantumRange*lightness;
-      return;
-    }
-  v=(lightness <= 0.5) ? (lightness*(1.0+saturation)) : (lightness+saturation-
-    lightness*saturation);
-  y=lightness+lightness-v;
-  x=y+(v-y)*(6.0*hue-(ssize_t) (6.0*hue));
-  z=v-(v-y)*(6.0*hue-(ssize_t) (6.0*hue));
-  switch ((ssize_t) (6.0*hue))
+  h=hue*360.0;
+  if (lightness <= 0.5)
+    c=2.0*lightness*saturation;
+  else
+    c=(2.0-2.0*lightness)*saturation;
+  min=lightness-0.5*c;
+  h-=360.0*floor(h/360.0);
+  h/=60.0;
+  x=c*(1.0-fabs(h-2.0*floor(h/2.0)-1.0));
+  switch ((int) floor(h))
   {
-    case 0: r=v; g=x; b=y; break;
-    case 1: r=z; g=v; b=y; break;
-    case 2: r=y; g=v; b=x; break;
-    case 3: r=y; g=z; b=v; break;
-    case 4: r=x; g=y; b=v; break;
-    case 5: r=v; g=y; b=z; break;
-    default: r=v; g=x; b=y; break;
+    case 0:
+    {
+      *red=QuantumRange*(min+c);
+      *green=QuantumRange*(min+x);
+      *blue=QuantumRange*min;
+      break;
+    }
+    case 1:
+    {
+      *red=QuantumRange*(min+x);
+      *green=QuantumRange*(min+c);
+      *blue=QuantumRange*min;
+      break;
+    }
+    case 2:
+    {
+      *red=QuantumRange*min;
+      *green=QuantumRange*(min+c);
+      *blue=QuantumRange*(min+x);
+      break;
+    }
+    case 3:
+    {
+      *red=QuantumRange*min;
+      *green=QuantumRange*(min+x);
+      *blue=QuantumRange*(min+c);
+      break;
+    }
+    case 4:
+    {
+      *red=QuantumRange*(min+x);
+      *green=QuantumRange*min;
+      *blue=QuantumRange*(min+c);
+      break;
+    }
+    case 5:
+    {
+      *red=QuantumRange*(min+c);
+      *green=QuantumRange*min;
+      *blue=QuantumRange*(min+x);
+      break;
+    }
+    default:
+    {
+      *red=0.0;
+      *green=0.0;
+      *blue=0.0;
+    }
   }
-  *red=(double) QuantumRange*r;
-  *green=(double) QuantumRange*g;
-  *blue=(double) QuantumRange*b;
 }
 
 /*
@@ -643,7 +675,7 @@ MagickPrivate void ConvertHWBToRGB(const double hue,const double whiteness,
     r,
     v;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -660,7 +692,7 @@ MagickPrivate void ConvertHWBToRGB(const double hue,const double whiteness,
       *blue=QuantumRange*v;
       return;
     }
-  i=(ssize_t) floor(6.0*hue);
+  i=CastDoubleToLong(floor(6.0*hue));
   f=6.0*hue-i;
   if ((i & 0x01) != 0)
     f=1.0-f;
@@ -710,14 +742,16 @@ MagickPrivate void ConvertHWBToRGB(const double hue,const double whiteness,
 */
 
 static inline void ConvertLCHabToXYZ(const double luma,const double chroma,
-  const double hue,double *X,double *Y,double *Z)
+  const double hue,const IlluminantType illuminant,double *X,double *Y,
+  double *Z)
 {
   ConvertLabToXYZ(luma,chroma*cos(hue*MagickPI/180.0),chroma*
-    sin(hue*MagickPI/180.0),X,Y,Z);
+    sin(hue*MagickPI/180.0),illuminant,X,Y,Z);
 }
 
 MagickPrivate void ConvertLCHabToRGB(const double luma,const double chroma,
-  const double hue,double *red,double *green,double *blue)
+  const double hue,const IlluminantType illuminant,double *red,double *green,
+  double *blue)
 {
   double
     X,
@@ -730,7 +764,8 @@ MagickPrivate void ConvertLCHabToRGB(const double luma,const double chroma,
   assert(red != (double *) NULL);
   assert(green != (double *) NULL);
   assert(blue != (double *) NULL);
-  ConvertLCHabToXYZ(100.0*luma,255.0*(chroma-0.5),360.0*hue,&X,&Y,&Z);
+  ConvertLCHabToXYZ(100.0*luma,255.0*(chroma-0.5),360.0*hue,illuminant,
+    &X,&Y,&Z);
   ConvertXYZToRGB(X,Y,Z,red,green,blue);
 }
 
@@ -763,14 +798,16 @@ MagickPrivate void ConvertLCHabToRGB(const double luma,const double chroma,
 */
 
 static inline void ConvertLCHuvToXYZ(const double luma,const double chroma,
-  const double hue,double *X,double *Y,double *Z)
+  const double hue,const IlluminantType illuminant,double *X,double *Y,
+  double *Z)
 {
   ConvertLuvToXYZ(luma,chroma*cos(hue*MagickPI/180.0),chroma*
-    sin(hue*MagickPI/180.0),X,Y,Z);
+    sin(hue*MagickPI/180.0),illuminant,X,Y,Z);
 }
 
 MagickPrivate void ConvertLCHuvToRGB(const double luma,const double chroma,
-  const double hue,double *red,double *green,double *blue)
+  const double hue,const IlluminantType illuminant,double *red,double *green,
+  double *blue)
 {
   double
     X,
@@ -783,7 +820,8 @@ MagickPrivate void ConvertLCHuvToRGB(const double luma,const double chroma,
   assert(red != (double *) NULL);
   assert(green != (double *) NULL);
   assert(blue != (double *) NULL);
-  ConvertLCHuvToXYZ(100.0*luma,255.0*(chroma-0.5),360.0*hue,&X,&Y,&Z);
+  ConvertLCHuvToXYZ(100.0*luma,255.0*(chroma-0.5),360.0*hue,illuminant,
+    &X,&Y,&Z);
   ConvertXYZToRGB(X,Y,Z,red,green,blue);
 }
 
@@ -1068,15 +1106,9 @@ MagickExport void ConvertRGBToHSL(const double red,const double green,
   const double blue,double *hue,double *saturation,double *lightness)
 {
   double
-    b,
-    delta,
-    g,
-    h,
-    l,
+    c,
     max,
-    min,
-    r,
-    s;
+    min;
 
   /*
     Convert RGB to HSL colorspace.
@@ -1084,30 +1116,34 @@ MagickExport void ConvertRGBToHSL(const double red,const double green,
   assert(hue != (double *) NULL);
   assert(saturation != (double *) NULL);
   assert(lightness != (double *) NULL);
-  r=QuantumScale*red;
-  g=QuantumScale*green;
-  b=QuantumScale*blue;
-  max=MagickMax(r,MagickMax(g,b));
-  min=MagickMin(r,MagickMin(g,b));
-  h=0.0;
-  s=0.0;
-  l=(min+max)/2.0;
-  delta=max-min;
-  if (delta != 0.0)
+  max=MagickMax(QuantumScale*red,MagickMax(QuantumScale*green,
+    QuantumScale*blue));
+  min=MagickMin(QuantumScale*red,MagickMin(QuantumScale*green,
+    QuantumScale*blue));
+  c=max-min;
+  *lightness=(max+min)/2.0;
+  if (c <= 0.0)
     {
-      s=delta/((l <= 0.5) ? (min+max) : (2.0-max-min));
-      if (r == max)
-        h=(g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
-      else
-        if (g == max)
-          h=(b == min ? 1.0+(max-r)/delta : 3.0-(max-b)/delta);
-        else
-          h=(r == min ? 3.0+(max-g)/delta : 5.0-(max-r)/delta);
-      h/=6.0;
+      *hue=0.0;
+      *saturation=0.0;
+      return;
     }
-  *hue=MagickMin(MagickMax(h,0.0),1.0);
-  *saturation=MagickMin(MagickMax(s,0.0),1.0);
-  *lightness=MagickMin(MagickMax(l,0.0),1.0);
+  if (fabs(max-QuantumScale*red) < MagickEpsilon)
+    {
+      *hue=(QuantumScale*green-QuantumScale*blue)/c;
+      if ((QuantumScale*green) < (QuantumScale*blue))
+        *hue+=6.0;
+    }
+  else
+    if (fabs(max-QuantumScale*green) < MagickEpsilon)
+      *hue=2.0+(QuantumScale*blue-QuantumScale*red)/c;
+    else
+      *hue=4.0+(QuantumScale*red-QuantumScale*green)/c;
+  *hue*=60.0/360.0;
+  if (*lightness <= 0.5)
+    *saturation=c*PerceptibleReciprocal(2.0*(*lightness));
+  else
+    *saturation=c*PerceptibleReciprocal(2.0-2.0*(*lightness));
 }
 
 /*
@@ -1176,7 +1212,7 @@ MagickPrivate void ConvertRGBToHSV(const double red,const double green,
     else
       *hue=4.0+(QuantumScale*red-QuantumScale*green)/c;
   *hue*=60.0/360.0;
-  *saturation=c/max;
+  *saturation=c*PerceptibleReciprocal(max);
 }
 
 /*
@@ -1243,6 +1279,46 @@ MagickPrivate void ConvertRGBToHWB(const double red,const double green,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C o n v e r t R G B T o L a b                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConvertRGBToLab() transforms a (red, green, blue) to a (L, a, b) triple.
+%
+%  The format of the ConvertRGBToLCHab method is:
+%
+%      void ConvertRGBToLCHab(const double red,const double green,
+%        const double blue,double *L,double *a,double *b)
+%
+%  A description of each parameter follows:
+%
+%    o red, green, blue: A Quantum value representing the red, green, and
+%      blue component of a pixel.
+%
+%    o L, a, b: A pointer to a double value representing a component of the
+%      Lab color space.
+%
+*/
+MagickPrivate void ConvertRGBToLab(const double red,const double green,
+  const double blue,const IlluminantType illuminant,double *L,double *a,
+  double *b)
+{
+  double
+    X,
+    Y,
+    Z;
+
+  ConvertRGBToXYZ(red,green,blue,&X,&Y,&Z);
+  ConvertXYZToLab(X,Y,Z,illuminant,L,a,b);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o n v e r t R G B T o L C H a b                                         %
 %                                                                             %
 %                                                                             %
@@ -1268,13 +1344,14 @@ MagickPrivate void ConvertRGBToHWB(const double red,const double green,
 */
 
 static inline void ConvertXYZToLCHab(const double X,const double Y,
-  const double Z,double *luma,double *chroma,double *hue)
+  const double Z,const IlluminantType illuminant,double *luma,double *chroma,
+  double *hue)
 {
   double
     a,
     b;
 
-  ConvertXYZToLab(X,Y,Z,luma,&a,&b);
+  ConvertXYZToLab(X,Y,Z,illuminant,luma,&a,&b);
   *chroma=hypot(255.0*(a-0.5),255.0*(b-0.5))/255.0+0.5;
   *hue=180.0*atan2(255.0*(b-0.5),255.0*(a-0.5))/MagickPI/360.0;
   if (*hue < 0.0)
@@ -1282,7 +1359,8 @@ static inline void ConvertXYZToLCHab(const double X,const double Y,
 }
 
 MagickPrivate void ConvertRGBToLCHab(const double red,const double green,
-  const double blue,double *luma,double *chroma,double *hue)
+  const double blue,const IlluminantType illuminant,double *luma,double *chroma,
+  double *hue)
 {
   double
     X,
@@ -1296,7 +1374,7 @@ MagickPrivate void ConvertRGBToLCHab(const double red,const double green,
   assert(chroma != (double *) NULL);
   assert(hue != (double *) NULL);
   ConvertRGBToXYZ(red,green,blue,&X,&Y,&Z);
-  ConvertXYZToLCHab(X,Y,Z,luma,chroma,hue);
+  ConvertXYZToLCHab(X,Y,Z,illuminant,luma,chroma,hue);
 }
 
 /*
@@ -1329,13 +1407,14 @@ MagickPrivate void ConvertRGBToLCHab(const double red,const double green,
 */
 
 static inline void ConvertXYZToLCHuv(const double X,const double Y,
-  const double Z,double *luma,double *chroma,double *hue)
+  const double Z,const IlluminantType illuminant,double *luma,double *chroma,
+  double *hue)
 {
   double
     u,
     v;
 
-  ConvertXYZToLuv(X,Y,Z,luma,&u,&v);
+  ConvertXYZToLuv(X,Y,Z,illuminant,luma,&u,&v);
   *chroma=hypot(354.0*u-134.0,262.0*v-140.0)/255.0+0.5;
   *hue=180.0*atan2(262.0*v-140.0,354.0*u-134.0)/MagickPI/360.0;
   if (*hue < 0.0)
@@ -1343,7 +1422,8 @@ static inline void ConvertXYZToLCHuv(const double X,const double Y,
 }
 
 MagickPrivate void ConvertRGBToLCHuv(const double red,const double green,
-  const double blue,double *luma,double *chroma,double *hue)
+  const double blue,const IlluminantType illuminant,double *luma,double *chroma,
+  double *hue)
 {
   double
     X,
@@ -1357,7 +1437,7 @@ MagickPrivate void ConvertRGBToLCHuv(const double red,const double green,
   assert(chroma != (double *) NULL);
   assert(hue != (double *) NULL);
   ConvertRGBToXYZ(red,green,blue,&X,&Y,&Z);
-  ConvertXYZToLCHuv(X,Y,Z,luma,chroma,hue);
+  ConvertXYZToLCHuv(X,Y,Z,illuminant,luma,chroma,hue);
 }
 
 /*
@@ -1509,7 +1589,7 @@ MagickPrivate double GenerateDifferentialNoise(RandomInfo *random_info,
       double
         poisson;
 
-      register ssize_t
+      ssize_t
         i;
 
       poisson=exp(-SigmaPoisson*QuantumScale*pixel);
@@ -1518,7 +1598,7 @@ MagickPrivate double GenerateDifferentialNoise(RandomInfo *random_info,
         beta=GetPseudoRandomValue(random_info);
         alpha*=beta;
       }
-      noise=(double) (QuantumRange*i/SigmaPoisson);
+      noise=(double) (QuantumRange*i*PerceptibleReciprocal(SigmaPoisson));
       break;
     }
     case RandomNoise:
@@ -1571,7 +1651,7 @@ MagickPrivate size_t GetOptimalKernelWidth1D(const double radius,
     normalize,
     value;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
