@@ -17,13 +17,13 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -66,7 +66,7 @@
 /*
   Define declarations.
 */
-#if defined(MAGICKCORE_HAVE_NEWLOCALE) || defined(MAGICKCORE_WINDOWS_SUPPORT)
+#if (defined(MAGICKCORE_HAVE_NEWLOCALE) || defined(MAGICKCORE_WINDOWS_SUPPORT)) && !defined(__MINGW32__)
 #  define MAGICKCORE_LOCALE_SUPPORT
 #endif
 #define LocaleFilename  "locale.xml"
@@ -169,7 +169,7 @@ static locale_t AcquireCLocale(void)
 
 static void *DestroyLocaleNode(void *locale_info)
 {
-  register LocaleInfo
+  LocaleInfo
     *p;
 
   p=(LocaleInfo *) locale_info;
@@ -185,16 +185,12 @@ static void *DestroyLocaleNode(void *locale_info)
 static SplayTreeInfo *AcquireLocaleSplayTree(const char *filename,
   const char *locale,ExceptionInfo *exception)
 {
-  MagickStatusType
-    status;
-
   SplayTreeInfo
     *cache;
 
   cache=NewSplayTree(CompareSplayTreeString,(void *(*)(void *)) NULL,
     DestroyLocaleNode);
-  status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     const StringInfo
       *option;
@@ -206,7 +202,7 @@ static SplayTreeInfo *AcquireLocaleSplayTree(const char *filename,
     option=(const StringInfo *) GetNextValueInLinkedList(options);
     while (option != (const StringInfo *) NULL)
     {
-      status&=LoadLocaleCache(cache,(const char *)
+      (void) LoadLocaleCache(cache,(const char *)
         GetStringInfoDatum(option),GetStringInfoPath(option),locale,0,
         exception);
       option=(const StringInfo *) GetNextValueInLinkedList(options);
@@ -218,7 +214,7 @@ static SplayTreeInfo *AcquireLocaleSplayTree(const char *filename,
         option=(const StringInfo *) GetNextValueInLinkedList(options);
         while (option != (const StringInfo *) NULL)
         {
-          status&=LoadLocaleCache(cache,(const char *)
+          (void) LoadLocaleCache(cache,(const char *)
             GetStringInfoDatum(option),GetStringInfoPath(option),locale,0,
             exception);
           option=(const StringInfo *) GetNextValueInLinkedList(options);
@@ -228,7 +224,7 @@ static SplayTreeInfo *AcquireLocaleSplayTree(const char *filename,
   }
 #endif
   if (GetNumberOfNodesInSplayTree(cache) == 0)
-    status&=LoadLocaleCache(cache,LocaleMap,"built-in",locale,0,
+    (void) LoadLocaleCache(cache,LocaleMap,"built-in",locale,0,
       exception);
   return(cache);
 }
@@ -255,13 +251,8 @@ static SplayTreeInfo *AcquireLocaleSplayTree(const char *filename,
 */
 static void DestroyCLocale(void)
 {
-#if defined(MAGICKCORE_HAVE_NEWLOCALE)
   if (c_locale != (locale_t) NULL)
     freelocale(c_locale);
-#elif defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
-  if (c_locale != (locale_t) NULL)
-    _free_locale(c_locale);
-#endif
   c_locale=(locale_t) NULL;
 }
 #endif
@@ -589,10 +580,10 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
   const LocaleInfo
     **messages;
 
-  register const LocaleInfo
+  const LocaleInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -665,7 +656,7 @@ extern "C" {
 
 static int LocaleTagCompare(const void *x,const void *y)
 {
-  register char
+  char
     **p,
     **q;
 
@@ -684,10 +675,10 @@ MagickExport char **GetLocaleList(const char *pattern,size_t *number_messages,
   char
     **messages;
 
-  register const LocaleInfo
+  const LocaleInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -919,7 +910,7 @@ static MagickBooleanType IsLocaleTreeInstantiated(ExceptionInfo *exception)
           char
             *locale;
 
-          register const char
+          const char
             *p;
 
           locale=(char *) NULL;
@@ -974,7 +965,7 @@ static MagickBooleanType IsLocaleTreeInstantiated(ExceptionInfo *exception)
 %
 */
 MagickExport double InterpretLocaleValue(const char *magick_restrict string,
-  char **magick_restrict sentinal)
+  char *magick_restrict *sentinal)
 {
   char
     *q;
@@ -1037,7 +1028,7 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
   const LocaleInfo
     **locale_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1109,7 +1100,7 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
 
 static void ChopLocaleComponents(char *path,const size_t components)
 {
-  register char
+  char
     *p;
 
   ssize_t
@@ -1167,7 +1158,7 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
   MagickStatusType
     status;
 
-  register char
+  char
     *p;
 
   size_t
@@ -1193,7 +1184,7 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
     /*
       Interpret XML.
     */
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MagickLocaleExtent);
@@ -1204,7 +1195,7 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
         {
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           while (isspace((int) ((unsigned char) *q)) != 0)
             q++;
         }
@@ -1217,7 +1208,7 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
         {
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           while (isspace((int) ((unsigned char) *q)) != 0)
             q++;
         }
@@ -1231,10 +1222,10 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MagickLocaleExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"locale") == 0)
             {
               if (LocaleCompare(locale,token) != 0)
@@ -1281,10 +1272,10 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
         while ((*token != '>') && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MagickLocaleExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         }
         continue;
       }
@@ -1306,10 +1297,10 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
         while ((*token != '>') && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MagickLocaleExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"name") == 0)
             {
               (void) ConcatenateMagickString(tag,token,MagickLocaleExtent);
@@ -1366,7 +1357,7 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
         (void) ConcatenateMagickString(tag,"/",MagickLocaleExtent);
         continue;
       }
-    GetNextToken(q,(const char **) NULL,extent,token);
+    (void) GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
   }
@@ -1407,32 +1398,23 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *cache,const char *xml,
 */
 MagickExport int LocaleCompare(const char *p,const char *q)
 {
-  if ((p == (char *) NULL) && (q == (char *) NULL))
-    return(0);
   if (p == (char *) NULL)
-    return(-1);
+    {
+      if (q == (char *) NULL)
+        return(0);
+      return(-1);
+    }
   if (q == (char *) NULL)
     return(1);
-#if defined(MAGICKCORE_HAVE_STRCASECMP)
-  return(strcasecmp(p,q));
-#else
   {
-    register int
-      c,
-      d;
+    const unsigned char
+      *r = (const unsigned char *) p,
+      *s = (const unsigned char *) q;
 
-    for ( ; ; )
-    {
-      c=(int) *((unsigned char *) p);
-      d=(int) *((unsigned char *) q);
-      if ((c == 0) || (AsciiMap[c] != AsciiMap[d]))
-        break;
-      p++;
-      q++;
-    }
-    return(AsciiMap[c]-(int) AsciiMap[d]);
+    for ( ; (*r != '\0') && (*s != '\0') && ((*r == *s) ||
+      (LocaleLowercase((int) *r) == LocaleLowercase((int) *s))); r++, s++);
+    return(LocaleLowercase((int) *r)-LocaleLowercase((int) *s));
   }
-#endif
 }
 
 /*
@@ -1460,12 +1442,45 @@ MagickExport int LocaleCompare(const char *p,const char *q)
 */
 MagickExport void LocaleLower(char *string)
 {
-  register char
+  char
     *q;
 
   assert(string != (char *) NULL);
   for (q=string; *q != '\0'; q++)
-    *q=(char) tolower((int) *q);
+    *q=(char) LocaleLowercase((int) *q);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   L o c a l e L o w e r c a s e                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  LocaleLowercase() convert to lowercase.
+%
+%  The format of the LocaleLowercase method is:
+%
+%      void LocaleLowercase(const int c)
+%
+%  A description of each parameter follows:
+%
+%    o If c is a uppercase letter, return its lowercase equivalent.
+%
+*/
+MagickExport int LocaleLowercase(const int c)
+{
+  if ((c == EOF) || (c != (unsigned char) c))
+    return(c);
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
+  if (c_locale != (locale_t) NULL)
+    return(tolower_l((int) ((unsigned char) c),c_locale));
+#endif
+  return(tolower((int) ((unsigned char) c)));
 }
 
 /*
@@ -1507,37 +1522,28 @@ MagickExport void LocaleLower(char *string)
 */
 MagickExport int LocaleNCompare(const char *p,const char *q,const size_t length)
 {
-  if ((p == (char *) NULL) && (q == (char *) NULL))
-    return(0);
   if (p == (char *) NULL)
-    return(-1);
+    {
+      if (q == (char *) NULL)
+        return(0);
+      return(-1);
+    }
   if (q == (char *) NULL)
     return(1);
-#if defined(MAGICKCORE_HAVE_STRNCASECMP)
-  return(strncasecmp(p,q,length));
-#else
-  {
-    register int
-      c,
-      d;
-
-    register size_t
-      i;
-
-    for (i=length; i != 0; i--)
-    {
-      c=(int) *((unsigned char *) p);
-      d=(int) *((unsigned char *) q);
-      if (AsciiMap[c] != AsciiMap[d])
-        return(AsciiMap[c]-(int) AsciiMap[d]);
-      if (c == 0)
-        return(0);
-      p++;
-      q++;
-    }
+  if (length == 0)
     return(0);
+  {
+    const unsigned char
+      *s = (const unsigned char *) p,
+      *t = (const unsigned char *) q;
+
+    size_t
+      n = length;
+
+    for (n--; (*s != '\0') && (*t != '\0') && (n != 0) && ((*s == *t) ||
+      (LocaleLowercase((int) *s) == LocaleLowercase((int) *t))); s++, t++, n--);
+    return(LocaleLowercase((int) *s)-LocaleLowercase((int) *t));
   }
-#endif
 }
 
 /*
@@ -1565,12 +1571,45 @@ MagickExport int LocaleNCompare(const char *p,const char *q,const size_t length)
 */
 MagickExport void LocaleUpper(char *string)
 {
-  register char
+  char
     *q;
 
   assert(string != (char *) NULL);
   for (q=string; *q != '\0'; q++)
-    *q=(char) toupper((int) *q);
+    *q=(char) LocaleUppercase((int) *q);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   L o c a l e U p p e r c a s e                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  LocaleUppercase() convert to uppercase.
+%
+%  The format of the LocaleUppercase method is:
+%
+%      void LocaleUppercase(const int c)
+%
+%  A description of each parameter follows:
+%
+%    o If c is a lowercase letter, return its uppercase equivalent.
+%
+*/
+MagickExport int LocaleUppercase(const int c)
+{
+  if (c == EOF)
+    return(c);
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
+  if (c_locale != (locale_t) NULL)
+    return(toupper_l((int) ((unsigned char) c),c_locale));
+#endif
+  return(toupper((int) ((unsigned char) c)));
 }
 
 /*
@@ -1595,6 +1634,9 @@ MagickPrivate MagickBooleanType LocaleComponentGenesis(void)
 {
   if (locale_semaphore == (SemaphoreInfo *) NULL)
     locale_semaphore=AcquireSemaphoreInfo();
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
+  (void) AcquireCLocale();
+#endif
   return(MagickTrue);
 }
 
