@@ -63,10 +63,34 @@ push(@$images,$example);
 print "Annotate...\n";
 $example=$model->Clone();
 $example->Label('Annotate');
-$example->Annotate(text=>'Magick',geometry=>'+0+20',font=>'Generic.ttf',
+$example->Annotate(font=>'Generic.ttf',text=>'Magick',geometry=>'+0+20',
   fill=>'gold',gravity=>'North',pointsize=>14);
 push(@$images,$example);
 
+print "Auto-gamma...\n";
+$example=$model->Clone();
+$example->Label('Auto Gamma');
+$example->AutoGamma();
+push(@$images,$example);
+
+print "Auto-level...\n";
+$example=$model->Clone();
+$example->Label('Auto Level');
+$example->AutoLevel();
+push(@$images,$example);
+
+print "Auto-threshold...\n";
+$example=$model->Clone();
+$example->Label('Auto Threshold');
+$example->AutoThreshold();
+push(@$images,$example);
+
+print "Blilateral Blur...\n";
+$example=$model->Clone();
+$example->Label('Blilateral Blur');
+$example->BilateralBlur('8x8');
+push(@$images,$example);
+
 print "Blur...\n";
 $example=$model->Clone();
 $example->Label('Blur');
@@ -79,6 +103,12 @@ $example->Label('Border');
 $example->Border(geometry=>'6x6',color=>'gold');
 push(@$images,$example);
 
+print "CLAHE...\n";
+$example=$model->Clone();
+$example->Label('CLAHE');
+$example->CLAHE('128x192+32+3');
+push(@$images,$example);
+
 print "Channel...\n";
 $example=$model->Clone();
 $example->Label('Channel');
@@ -88,13 +118,26 @@ push(@$images,$example);
 print "Charcoal...\n";
 $example=$model->Clone();
 $example->Label('Charcoal');
-$example->Charcoal('0x1');
+$example->Charcoal('2x1');
+push(@$images,$example);
+
+print "ColorMatrix...\n";
+$example=$model->Clone();
+$example->Label('ColorMatrix');
+$example->ColorMatrix([1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0.5, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1]);
+push(@$images,$example);
+
+print "Colorspace...\n";
+$example=$model->Clone();
+$example->Label('Colorspace');
+$example->Colorspace('Lab');
 push(@$images,$example);
 
 print "Composite...\n";
 $example=$model->Clone();
 $example->Label('Composite');
 $example->Composite(image=>$smile,compose=>'over',geometry=>'+35+65');
+$example->Clamp();
 push(@$images,$example);
 
 print "Contrast...\n";
@@ -112,7 +155,7 @@ push(@$images,$example);
 print "Convolve...\n";
 $example=$model->Clone();
 $example->Label('Convolve');
-$example->Convolve([1, 1, 1, 1, 4, 1, 1, 1, 1]);
+$example->Convolve([0.125, 0.125, 0.125, 0.125, 0.5, 0.125, 0.125, 0.125, 0.125]);
 push(@$images,$example);
 
 print "Crop...\n";
@@ -131,7 +174,7 @@ push(@$images,$example);
 print "Distort...\n";
 $example=$model->Clone();
 $example->Label('Distort');
-$example->Distort(type=>'arc',points=>[60],'virtual-pixel'=>'white');
+$example->Distort(method=>'arc',points=>[60],'virtual-pixel'=>'white');
 push(@$images,$example);
 
 print "Draw...\n";
@@ -144,7 +187,8 @@ push(@$images,$example);
 print "Detect Edges...\n";
 $example=$model->Clone();
 $example->Label('Detect Edges');
-$example->Edge();
+$example->Edge('2x0.5');
+$example->Clamp();
 push(@$images,$example);
 
 print "Emboss...\n";
@@ -153,13 +197,19 @@ $example->Label('Emboss');
 $example->Emboss('0x1');
 push(@$images,$example);
 
+print "Encipher...\n";
+$example=$model->Clone();
+$example->Label('Encipher');
+$example->Encipher('Magick');
+push(@$images,$example);
+
 print "Equalize...\n";
 $example=$model->Clone();
 $example->Label('Equalize');
 $example->Equalize();
 push(@$images,$example);
 
-print "Implode...\n";
+print "Explode...\n";
 $example=$model->Clone();
 $example->Label('Explode');
 $example->Implode(-1);
@@ -211,7 +261,7 @@ push(@$images,$gradient);
 print "Grayscale...\n";
 $example=$model->Clone();
 $example->Label('Grayscale');
-$example->Quantize(colorspace=>'gray');
+$example->Set(type=>'grayscale');
 push(@$images,$example);
 
 print "Implode...\n";
@@ -220,16 +270,35 @@ $example->Label('Implode');
 $example->Implode(0.5);
 push(@$images,$example);
 
+print "Kuwahara...\n";
+$example=$model->Clone();
+$example->Label('Kuwahara');
+$example->Kuwahara('0x1');
+push(@$images,$example);
+
 print "Level...\n";
 $example=$model->Clone();
 $example->Label('Level');
-$example->Level('20%');
+$example->Level('20%x');
+$example->Clamp();
+push(@$images,$example);
+
+print "Linear stretch...\n";
+$example=$model->Clone();
+$example->Label('Linear Stretch');
+$example->LinearStretch('5x5');
 push(@$images,$example);
 
 print "Median Filter...\n";
 $example=$model->Clone();
 $example->Label('Median Filter');
-$example->MedianFilter();
+$example->MedianFilter('4x4');
+push(@$images,$example);
+
+print "Mode...\n";
+$example=$model->Clone();
+$example->Label('Mode');
+$example->Mode('4x4');
 push(@$images,$example);
 
 print "Modulate...\n";
@@ -240,10 +309,17 @@ push(@$images,$example);
 $example=$model->Clone();
 
 print "Monochrome...\n";
+$example=$model->Clone();
 $example->Label('Monochrome');
 $example->Quantize(colorspace=>'gray',colors=>2,dither=>'false');
 push(@$images,$example);
 
+print "Morphology...\n";
+$example=$model->Clone();
+$example->Label('Morphology');
+$example->Morphology(method=>'Dilate',kernel=>'Diamond',iterations=>2);
+push(@$images,$example);
+
 print "Motion Blur...\n";
 $example=$model->Clone();
 $example->Label('Motion Blur');
@@ -265,7 +341,7 @@ push(@$images,$example);
 print "Oil Paint...\n";
 $example=$model->Clone();
 $example->Label('Oil Paint');
-$example->OilPaint();
+$example->OilPaint('2x0.5');
 push(@$images,$example);
 
 print "Plasma...\n";
@@ -279,7 +355,13 @@ push(@$images,$plasma);
 print "Polaroid...\n";
 $example=$model->Clone();
 $example->Label('Polaroid');
-$example->Polaroid(caption=>'Magick',rotate=>-5.0,gravity=>'center');
+$example->Polaroid(caption=>'Magick',angle=>-5.0,gravity=>'center');
+push(@$images,$example);
+
+print "Posterize...\n";
+$example=$model->Clone();
+$example->Label('Posterize');
+$example->Posterize(5);
 push(@$images,$example);
 
 print "Quantize...\n";
@@ -288,10 +370,10 @@ $example->Label('Quantize');
 $example->Quantize();
 push(@$images,$example);
 
-print "Radial Blur...\n";
+print "Rotational Blur...\n";
 $example=$model->Clone();
-$example->Label('Radial Blur');
-$example->RadialBlur(10);
+$example->Label('Rotational Blur');
+$example->RotationalBlur(10);
 push(@$images,$example);
 
 print "Raise...\n";
@@ -300,16 +382,10 @@ $example->Label('Raise');
 $example->Raise('10x10');
 push(@$images,$example);
 
-print "Recolor...\n";
-$example=$model->Clone();
-$example->Label('Recolor');
-$example->Recolor([1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0.5, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1]);
-push(@$images,$example);
-
 print "Reduce Noise...\n";
 $example=$model->Clone();
 $example->Label('Reduce Noise');
-$example->ReduceNoise();
+$example->ReduceNoise('2x2');
 push(@$images,$example);
 
 print "Resize...\n";
@@ -358,6 +434,7 @@ print "Sharpen...\n";
 $example=$model->Clone();
 $example->Label('Sharpen');
 $example->Sharpen('0.0x1.0');
+$example->Clamp();
 push(@$images,$example);
 
 print "Shave...\n";
@@ -403,10 +480,17 @@ $example->Label('Swirl');
 $example->Swirl(90);
 push(@$images,$example);
 
+print "Tint...\n";
+$example=$model->Clone();
+$example->Label('Tint');
+$example->Tint('wheat');
+push(@$images,$example);
+
 print "Unsharp Mask...\n";
 $example=$model->Clone();
 $example->Label('Unsharp Mask');
 $example->UnsharpMask('0.0x1.0');
+$example->Clamp();
 push(@$images,$example);
 
 print "Vignette...\n";
@@ -420,14 +504,20 @@ $example=$model->Clone();
 $example->Label('Wave');
 $example->Wave('25x150');
 push(@$images,$example);
+
+print "WaveletDenoise...\n";
+$example=$model->Clone();
+$example->Label('Wavelet Denoise');
+$example->WaveletDenoise('5%');
+push(@$images,$example);
+
 #
 # Create image montage.
 #
 print "Montage...\n";
-$montage=$images->Montage(geometry=>'128x160+8+4>',gravity=>'Center',
-  tile=>'5x+10+200',compose=>'over',background=>'#ffffff',
-  font=>'Generic.ttf',pointsize=>18,fill=>'#600',stroke=>'none',
-	shadow=>'true');
+$montage=$images->Montage(font=>'Generic.ttf',geometry=>'140x160+8+4>',
+  gravity=>'Center',tile=>'5x+10+200',compose=>'over',background=>'#ffffff',
+  pointsize=>18,fill=>'#600',stroke=>'none',shadow=>'true');
 
 $logo=Image::Magick->new();
 $logo->Read('logo:');
