@@ -13,17 +13,17 @@
 %                             Read Text Caption.                              %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,30 +39,30 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/annotate.h"
-#include "magick/artifact.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/composite-private.h"
-#include "magick/draw.h"
-#include "magick/draw-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/annotate.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/composite-private.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/draw-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/option.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -91,13 +91,28 @@
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static inline void AdjustTypeMetricBounds(TypeMetric *metrics)
+{
+  double
+    x1;
+
+  if (metrics->bounds.x1 >= 0.0)
+    {
+      metrics->bounds.x1=0.0;
+      return;
+    }
+  x1=ceil(-metrics->bounds.x1+0.5);
+  metrics->width+=x1+x1;
+  metrics->bounds.x1=x1;
+}
+
 static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
     *caption,
-    geometry[MaxTextExtent],
-    *property,
+    geometry[MagickPathExtent],
     *text;
 
   const char
@@ -114,13 +129,13 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     split,
     status;
 
-  register ssize_t
-    i;
-
   size_t
     height,
     width;
 
+  ssize_t
+    i;
+
   TypeMetric
     metrics;
 
@@ -128,112 +143,169 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     Initialize Image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   (void) ResetImagePage(image,"0x0+0+0");
+  if ((image->columns != 0) && (image->rows != 0))
+    (void) SetImageBackgroundColor(image,exception);
   /*
     Format caption.
   */
   option=GetImageOption(image_info,"filename");
   if (option == (const char *) NULL)
-    property=InterpretImageProperties(image_info,image,image_info->filename);
+    caption=InterpretImageProperties((ImageInfo *) image_info,image,
+      image_info->filename,exception);
   else
     if (LocaleNCompare(option,"caption:",8) == 0)
-      property=InterpretImageProperties(image_info,image,option+8);
+      caption=InterpretImageProperties((ImageInfo *) image_info,image,option+8,
+        exception);
     else
-      property=InterpretImageProperties(image_info,image,option);
-  (void) SetImageProperty(image,"caption",property);
-  property=DestroyString(property);
-  caption=ConstantString(GetImageProperty(image,"caption"));
+      caption=InterpretImageProperties((ImageInfo *) image_info,image,option,
+        exception);
+  if (caption == (char *) NULL)
+    return(DestroyImageList(image));
+  (void) SetImageProperty(image,"caption",caption,exception);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+  width=(size_t) floor(0.5*draw_info->pointsize*strlen(caption)+0.5);
+  if (AcquireMagickResource(WidthResource,width) == MagickFalse)
+    {
+      caption=DestroyString(caption);
+      draw_info=DestroyDrawInfo(draw_info);
+      ThrowReaderException(ImageError,"WidthOrHeightExceedsLimit");
+    }
   (void) CloneString(&draw_info->text,caption);
   gravity=GetImageOption(image_info,"gravity");
   if (gravity != (char *) NULL)
     draw_info->gravity=(GravityType) ParseCommandOption(MagickGravityOptions,
       MagickFalse,gravity);
-  split=MagickFalse;
+  split=IsStringTrue(GetImageOption(image_info,"caption:split"));
+  status=MagickTrue;
+  (void) memset(&metrics,0,sizeof(metrics));
   if (image->columns == 0)
     {
       text=AcquireString(caption);
-      i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+      i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
+      AdjustTypeMetricBounds(&metrics);
       (void) CloneString(&draw_info->text,text);
       text=DestroyString(text);
-      (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-        -metrics.bounds.x1,metrics.ascent);
+      (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+        metrics.bounds.x1,metrics.ascent);
       if (draw_info->gravity == UndefinedGravity)
         (void) CloneString(&draw_info->geometry,geometry);
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-      width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-      image->columns=width;
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+      AdjustTypeMetricBounds(&metrics);
+      image->columns=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
     }
   if (image->rows == 0)
     {
       split=MagickTrue;
       text=AcquireString(caption);
-      i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+      i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
+      AdjustTypeMetricBounds(&metrics);
       (void) CloneString(&draw_info->text,text);
       text=DestroyString(text);
-      (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-        -metrics.bounds.x1,metrics.ascent);
+      (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+        metrics.bounds.x1,metrics.ascent);
       if (draw_info->gravity == UndefinedGravity)
         (void) CloneString(&draw_info->geometry,geometry);
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+      AdjustTypeMetricBounds(&metrics);
       image->rows=(size_t) ((i+1)*(metrics.ascent-metrics.descent+
         draw_info->interline_spacing+draw_info->stroke_width)+0.5);
     }
-  if (fabs(image_info->pointsize) < MagickEpsilon)
+  if (status != MagickFalse)
+    status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
+    { 
+      caption=DestroyString(caption);
+      draw_info=DestroyDrawInfo(draw_info);
+      return(DestroyImageList(image));
+    }
+  if (SetImageBackgroundColor(image,exception) == MagickFalse)
+    {
+      caption=DestroyString(caption);
+      draw_info=DestroyDrawInfo(draw_info);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  if ((fabs(image_info->pointsize) < MagickEpsilon) && (strlen(caption) > 0))
     {
       double
         high,
         low;
 
+      ssize_t
+        n;
+
       /*
         Auto fit text into bounding box.
       */
-      for ( ; ; draw_info->pointsize*=2.0)
-      {
-        text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
-        (void) CloneString(&draw_info->text,text);
-        text=DestroyString(text);
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-          -metrics.bounds.x1,metrics.ascent);
-        if (draw_info->gravity == UndefinedGravity)
-          (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-        (void) status;
-        width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-        height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if ((image->columns != 0) && (image->rows != 0))
+      option=GetImageOption(image_info,"caption:max-pointsize");
+      if (option != (const char*) NULL)
+        {
+          high=StringToDouble(option,(char**) NULL);
+          if (high < 1.0)
+            high=1.0;
+          high+=1.0;
+        }
+      else
+        {
+          for (n=0; n < 32; n++, draw_info->pointsize*=2.0)
           {
-            if ((width >= image->columns) && (height >= image->rows))
+            text=AcquireString(caption);
+            i=FormatMagickCaption(image,draw_info,split,&metrics,&text,
+              exception);
+            AdjustTypeMetricBounds(&metrics);
+            (void) CloneString(&draw_info->text,text);
+            text=DestroyString(text);
+            (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+              metrics.bounds.x1,metrics.ascent);
+            if (draw_info->gravity == UndefinedGravity)
+              (void) CloneString(&draw_info->geometry,geometry);
+            status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+            if (status == MagickFalse)
               break;
+            AdjustTypeMetricBounds(&metrics);
+            width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+            height=(size_t) floor(metrics.height+draw_info->interline_spacing+
+              draw_info->stroke_width+0.5);
+            if ((image->columns != 0) && (image->rows != 0))
+              {
+                if ((width >= image->columns) && (height >= image->rows))
+                  break;
+              }
+            else
+              if (((image->columns != 0) && (width >= image->columns)) ||
+                  ((image->rows != 0) && (height >= image->rows)))
+                break;
           }
-        else
-          if (((image->columns != 0) && (width >= image->columns)) ||
-              ((image->rows != 0) && (height >= image->rows)))
-            break;
-      }
-      high=draw_info->pointsize;
+          high=draw_info->pointsize;
+        }
       for (low=1.0; (high-low) > 0.5; )
       {
         draw_info->pointsize=(low+high)/2.0;
         text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+        i=FormatMagickCaption(image,draw_info,split,&metrics,&text,
+          exception);
+        AdjustTypeMetricBounds(&metrics);
         (void) CloneString(&draw_info->text,text);
         text=DestroyString(text);
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-          -metrics.bounds.x1,metrics.ascent);
+        (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+          metrics.bounds.x1,metrics.ascent);
         if (draw_info->gravity == UndefinedGravity)
           (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+        status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+        if (status == MagickFalse)
+          break;
+        AdjustTypeMetricBounds(&metrics);
         width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-        height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
+        height=(size_t) floor(metrics.height+draw_info->interline_spacing+
+          draw_info->stroke_width+0.5);
         if ((image->columns != 0) && (image->rows != 0))
           {
             if ((width < image->columns) && (height < image->rows))
@@ -248,38 +320,27 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
           else
             high=draw_info->pointsize-0.5;
       }
-      draw_info->pointsize=(low+high)/2.0-0.5;
-    }
-  (void) CloneString(&draw_info->text,caption);
-  i=FormatMagickCaption(image,draw_info,split,&metrics,&caption);
-  if (SetImageBackgroundColor(image) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      image=DestroyImageList(image);
-      return((Image *) NULL);
+      draw_info->pointsize=floor((low+high)/2.0-0.5);
     }
   /*
     Draw caption.
   */
+  i=FormatMagickCaption(image,draw_info,split,&metrics,&caption,exception);
+  AdjustTypeMetricBounds(&metrics);
   (void) CloneString(&draw_info->text,caption);
-  status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-  if ((draw_info->gravity != UndefinedGravity) &&
-      (draw_info->direction != RightToLeftDirection))
-    image->page.x=(ssize_t) (metrics.bounds.x1-draw_info->stroke_width/2.0);
-  else
-    {
-      (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-        -metrics.bounds.x1+draw_info->stroke_width/2.0,metrics.ascent+
-        draw_info->stroke_width/2.0);
-      if (draw_info->direction == RightToLeftDirection)
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-          image->columns-(metrics.bounds.x2+draw_info->stroke_width/2.0),
-          metrics.ascent+draw_info->stroke_width/2.0);
-      draw_info->geometry=AcquireString(geometry);
-    }
-  status=AnnotateImage(image,draw_info);
-  draw_info=DestroyDrawInfo(draw_info);
   caption=DestroyString(caption);
+  (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+    (draw_info->direction == RightToLeftDirection ? (double) image->columns-
+    metrics.bounds.x2 : metrics.bounds.x1),(draw_info->gravity ==
+    UndefinedGravity ? MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0));
+  (void) CloneString(&draw_info->geometry,geometry);
+  status=AnnotateImage(image,draw_info,exception);
+  if (image_info->pointsize == 0.0)
+    (void) FormatImageProperty(image,"caption:pointsize","%.*g",
+      GetMagickPrecision(),draw_info->pointsize);
+  (void) FormatImageProperty(image,"caption:lines","%.*g",GetMagickPrecision(),
+    (double) (i+1));
+  draw_info=DestroyDrawInfo(draw_info);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
@@ -316,11 +377,9 @@ ModuleExport size_t RegisterCAPTIONImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("CAPTION");
+  entry=AcquireMagickInfo("CAPTION","CAPTION","Caption");
   entry->decoder=(DecodeImageHandler *) ReadCAPTIONImage;
-  entry->description=ConstantString("Caption");
-  entry->adjoin=MagickFalse;
-  entry->module=ConstantString("CAPTION");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
