@@ -9,7 +9,7 @@
 %                              CCC   UUU     T                                %
 %                                                                             %
 %                                                                             %
-%                         Read DR Halo Image Format.                          %
+%                         Read DR Halo Image Format                           %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -47,23 +47,31 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/color.h"
-#include "magick/color-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colormap.h"
+#include "MagickCore/colormap-private.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
+#include "MagickCore/utility-private.h"
+#include "coders/coders-private.h"
 
 typedef struct
 {
@@ -89,183 +97,197 @@ typedef struct
 } CUTPalHeader;
 
 
-static void InsertRow(long depth,unsigned char *p,long y,Image *image)
+static MagickBooleanType InsertRow(Image *image,ssize_t bpp,unsigned char *p,
+  ssize_t y,ExceptionInfo *exception)
 {
-  unsigned long bit; long x;
-  register PixelPacket *q;
-  IndexPacket index;
-  register IndexPacket *indexes;
+  int
+    bit;
 
+  Quantum
+    index;
 
-  index=(IndexPacket) 0;
-  switch (depth)
+  Quantum
+    *q;
+
+  ssize_t
+    x;
+
+  q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+  if (q == (Quantum *) NULL)
+    return(MagickFalse);
+  switch (bpp)
     {
     case 1:  /* Convert bitmap scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-7); x+=8)
+        for (x=0; x < ((ssize_t) image->columns-7); x+=8)
+        {
+          for (bit=0; bit < 8; bit++)
           {
-            for (bit=0; bit < 8; bit++)
-              {
-                index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
-                indexes[x+bit]=index;
-                *q++=image->colormap[(long) index];
-              }
-            p++;
+            index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
           }
+          p++;
+        }
         if ((image->columns % 8) != 0)
           {
-            for (bit=0; bit < (image->columns % 8); bit++)
-              {
-                index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
-                indexes[x+bit]=index;
-                *q++=image->colormap[(long) index];
-              }
+            for (bit=0; bit < (ssize_t) (image->columns % 8); bit++)
+            {
+              index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+              SetPixelIndex(image,index,q);
+              if (index < image->colors)
+                SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+              q+=GetPixelChannels(image);
+            }
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-        /*            if (image->previous == (Image *) NULL)
-                      if (QuantumTick(y,image->rows) != MagickFalse)
-                      ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
     case 2:  /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
-          {
-            index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-            index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-            index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-            index=ConstrainColormapIndex(image,(*p) & 0x3);
-            indexes[x+1]=index;
-            *q++=image->colormap[(long) index];
+        for (x=0; x < ((ssize_t) image->columns-3); x+=4)
+        {
+            index=ConstrainColormapIndex(image,(*p >> 6) & 0x3,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
+            index=ConstrainColormapIndex(image,(*p >> 4) & 0x3,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
+            index=ConstrainColormapIndex(image,(*p >> 2) & 0x3,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
+            index=ConstrainColormapIndex(image,(*p) & 0x3,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
             p++;
-          }
-        if ((image->columns % 4) != 0)
+        }
+       if ((image->columns % 4) != 0)
           {
-            index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-            if ((image->columns % 4) >= 1)
-
+            index=ConstrainColormapIndex(image,(*p >> 6) & 0x3,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
+            if ((image->columns % 4) > 1)
               {
-                index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
-                indexes[x]=index;
-                *q++=image->colormap[(long) index];
-                if ((image->columns % 4) >= 2)
-
+                index=ConstrainColormapIndex(image,(*p >> 4) & 0x3,exception);
+                SetPixelIndex(image,index,q);
+                if (index < image->colors)
+                  SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+                q+=GetPixelChannels(image);
+                if ((image->columns % 4) > 2)
                   {
-                    index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
-                    indexes[x]=index;
-                    *q++=image->colormap[(long) index];
+                    index=ConstrainColormapIndex(image,(*p >> 2) & 0x3,
+                      exception);
+                    SetPixelIndex(image,index,q);
+                    if (index < image->colors)
+                      SetPixelViaPixelInfo(image,image->colormap+(ssize_t)
+                        index,q);
+                    q+=GetPixelChannels(image);
                   }
               }
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-        /*         if (image->previous == (Image *) NULL)
-                   if (QuantumTick(y,image->rows) != MagickFalse)
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
 
     case 4:  /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        for (x=0; x < ((ssize_t) image->columns-1); x+=2)
           {
-            index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-            index=ConstrainColormapIndex(image,(*p) & 0xf);
-            indexes[x+1]=index;
-            *q++=image->colormap[(long) index];
+            index=ConstrainColormapIndex(image,(*p >> 4) & 0x0f,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
+            q+=GetPixelChannels(image);
+            index=ConstrainColormapIndex(image,(*p) & 0x0f,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
             p++;
+            q+=GetPixelChannels(image);
           }
         if ((image->columns % 2) != 0)
           {
-            index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            index=ConstrainColormapIndex(image,(*p >> 4) & 0x0f,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
             p++;
+            q+=GetPixelChannels(image);
           }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-        /*         if (image->previous == (Image *) NULL)
-                   if (QuantumTick(y,image->rows) != MagickFalse)
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
     case 8: /* Convert PseudoColor scanline. */
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL) break;
-        indexes=GetIndexes(image);
-
-        for (x=0; x < (long) image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
           {
-            index=ConstrainColormapIndex(image,*p);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
+            index=ConstrainColormapIndex(image,*p,exception);
+            SetPixelIndex(image,index,q);
+            if (index < image->colors)
+              SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
             p++;
+            q+=GetPixelChannels(image);
           }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-        /*           if (image->previous == (Image *) NULL)
-                     if (QuantumTick(y,image->rows) != MagickFalse)
-                     ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
       }
       break;
 
+    case 24:     /*  Convert DirectColor scanline.  */
+      for (x=0; x < (ssize_t) image->columns; x++)
+        {
+          SetPixelRed(image,ScaleCharToQuantum(*p++),q);
+          SetPixelGreen(image,ScaleCharToQuantum(*p++),q);
+          SetPixelBlue(image,ScaleCharToQuantum(*p++),q);
+          q+=GetPixelChannels(image);
+        }
+      break;
     }
+  if (!SyncAuthenticPixels(image,exception))
+    return(MagickFalse);
+  return(MagickTrue);
 }
 
 /*
    Compute the number of colors in Grayed R[i]=G[i]=B[i] image
 */
-static int GetCutColors(Image *image)
+static int GetCutColors(Image *image,ExceptionInfo *exception)
 {
-  long
-    x,
-    y;
-
-  PixelPacket
-    *q;
-
   Quantum
     intensity,
     scale_intensity;
 
+  Quantum
+    *q;
+
+  ssize_t
+    x,
+    y;
+
   intensity=0;
   scale_intensity=ScaleCharToQuantum(16);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
-    for (x=0; x < (long) image->columns; x++)
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
+      break;
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (intensity < q->red)
-        intensity=q->red;
+      if (intensity < GetPixelRed(image,q))
+        intensity=GetPixelRed(image,q);
       if (intensity >= scale_intensity)
         return(255);
-      q++;
+      q+=GetPixelChannels(image);
     }
   }
   if (intensity < ScaleCharToQuantum(2))
@@ -303,6 +325,15 @@ static int GetCutColors(Image *image)
 */
 static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define ThrowCUTReaderException(severity,tag) \
+{ \
+  if (palette != NULL) \
+    palette=DestroyImage(palette); \
+  if (clone_info != NULL) \
+    clone_info=DestroyImageInfo(clone_info); \
+  ThrowReaderException(severity,tag); \
+}
+
   Image *image,*palette;
   ImageInfo *clone_info;
   MagickBooleanType status;
@@ -310,30 +341,27 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickOffsetType
     offset;
 
-  unsigned long EncodedByte;
+  size_t EncodedByte;
   unsigned char RunCount,RunValue,RunCountMasked;
   CUTHeader  Header;
   CUTPalHeader PalHeader;
-  long depth;
-  long i,j;
-  long ldblk;
+  ssize_t depth;
+  ssize_t i,j;
+  ssize_t ldblk;
   unsigned char *BImgBuff=NULL,*ptrB;
-  PixelPacket *q;
-
-  ssize_t
-    count;
+  Quantum *q;
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -350,7 +378,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Header.Reserved=ReadBlobLSBShort(image);
 
   if (Header.Width==0 || Header.Height==0 || Header.Reserved!=0)
-    CUT_KO:  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    CUT_KO:  ThrowCUTReaderException(CorruptImageError,"ImproperImageHeader");
 
   /*---This code checks first line of image---*/
   EncodedByte=ReadBlobLSBShort(image);
@@ -360,13 +388,13 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   while((int) RunCountMasked!=0)  /*end of line?*/
     {
       i=1;
-      if((int) RunCount<0x80) i=(long) RunCountMasked;
+      if((int) RunCount<0x80) i=(ssize_t) RunCountMasked;
       offset=SeekBlob(image,TellBlob(image)+i,SEEK_SET);
       if (offset < 0)
-        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        ThrowCUTReaderException(CorruptImageError,"ImproperImageHeader");
       if(EOFBlob(image) != MagickFalse) goto CUT_KO;  /*wrong data*/
       EncodedByte-=i+1;
-      ldblk+=(long) RunCountMasked;
+      ldblk+=(ssize_t) RunCountMasked;
 
       RunCount=(unsigned char) ReadBlobByte(image);
       if(EOFBlob(image) != MagickFalse)  goto CUT_KO;  /*wrong data: unexpected eof in line*/
@@ -383,20 +411,18 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=Header.Width;
   image->rows=Header.Height;
   image->depth=8;
-  image->colors=(unsigned long) (GetQuantumRange(1UL*i)+1);
+  image->colors=(size_t) (GetQuantumRange(1UL*i)+1);
 
-  if (image_info->ping) goto Finish;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+  if (image_info->ping != MagickFalse) goto Finish;
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
 
   /* ----- Do something with palette ----- */
   if ((clone_info=CloneImageInfo(image_info)) == NULL) goto NoPalette;
 
 
-  i=(long) strlen(clone_info->filename);
+  i=(ssize_t) strlen(clone_info->filename);
   j=i;
   while(--i>0)
     {
@@ -413,15 +439,15 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
 
   (void) CopyMagickString(clone_info->filename+i,".PAL",(size_t)
-    (MaxTextExtent-i));
-  if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
+    (MagickPathExtent-i));
+  if((clone_info->file=fopen_utf8(clone_info->filename,"rb"))==NULL)
     {
       (void) CopyMagickString(clone_info->filename+i,".pal",(size_t)
-        (MaxTextExtent-i));
-      if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
+        (MagickPathExtent-i));
+      if((clone_info->file=fopen_utf8(clone_info->filename,"rb"))==NULL)
         {
           clone_info->filename[i]='\0';
-          if((clone_info->file=OpenMagickStream(clone_info->filename,"rb"))==NULL)
+          if((clone_info->file=fopen_utf8(clone_info->filename,"rb"))==NULL)
             {
               clone_info=DestroyImageInfo(clone_info);
               clone_info=NULL;
@@ -430,7 +456,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
 
-  if( (palette=AcquireImage(clone_info))==NULL ) goto NoPalette;
+  if( (palette=AcquireImage(clone_info,exception))==NULL ) goto NoPalette;
   status=OpenBlob(clone_info,palette,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -443,7 +469,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   if(palette!=NULL)
     {
-      count=ReadBlob(palette,2,(unsigned char *) PalHeader.FileId);
+      (void) ReadBlob(palette,2,(unsigned char *) PalHeader.FileId);
       if(strncmp(PalHeader.FileId,"AH",2) != 0) goto ErasePalette;
       PalHeader.Version=ReadBlobLSBShort(palette);
       PalHeader.Size=ReadBlobLSBShort(palette);
@@ -455,11 +481,13 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       PalHeader.MaxRed=ReadBlobLSBShort(palette);
       PalHeader.MaxGreen=ReadBlobLSBShort(palette);
       PalHeader.MaxBlue=ReadBlobLSBShort(palette);
-      count=ReadBlob(palette,20,(unsigned char *) PalHeader.PaletteId);
+      (void) ReadBlob(palette,20,(unsigned char *) PalHeader.PaletteId);
+      if (EOFBlob(image))
+        ThrowCUTReaderException(CorruptImageError,"UnexpectedEndOfFile");
 
       if(PalHeader.MaxIndex<1) goto ErasePalette;
       image->colors=PalHeader.MaxIndex+1;
-      if (AcquireImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
+      if (AcquireImageColormap(image,image->colors,exception) == MagickFalse) goto NoMemory;
 
       if(PalHeader.MaxRed==0) PalHeader.MaxRed=(unsigned int) QuantumRange;  /*avoid division by 0*/
       if(PalHeader.MaxGreen==0) PalHeader.MaxGreen=(unsigned int) QuantumRange;
@@ -467,7 +495,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       for(i=0;i<=(int) PalHeader.MaxIndex;i++)
         {      /*this may be wrong- I don't know why is palette such strange*/
-          j=(long) TellBlob(palette);
+          j=(ssize_t) TellBlob(palette);
           if((j % 512)>512-6)
             {
               j=((j / 512)+1)*512;
@@ -476,26 +504,28 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 ThrowReaderException(CorruptImageError,"ImproperImageHeader");
             }
           image->colormap[i].red=(Quantum) ReadBlobLSBShort(palette);
-          if(QuantumRange!=(Quantum) PalHeader.MaxRed)
+          if (QuantumRange != (Quantum) PalHeader.MaxRed)
             {
-              image->colormap[i].red=RoundToQuantum(((double)
+              image->colormap[i].red=ClampToQuantum(((double)
                 image->colormap[i].red*QuantumRange+(PalHeader.MaxRed>>1))/
                 PalHeader.MaxRed);
             }
           image->colormap[i].green=(Quantum) ReadBlobLSBShort(palette);
-          if(QuantumRange!=(Quantum) PalHeader.MaxGreen)
+          if (QuantumRange != (Quantum) PalHeader.MaxGreen)
             {
-              image->colormap[i].green=RoundToQuantum
+              image->colormap[i].green=ClampToQuantum
                 (((double) image->colormap[i].green*QuantumRange+(PalHeader.MaxGreen>>1))/PalHeader.MaxGreen);
             }
           image->colormap[i].blue=(Quantum) ReadBlobLSBShort(palette);
-          if(QuantumRange!=(Quantum) PalHeader.MaxBlue)
+          if (QuantumRange != (Quantum) PalHeader.MaxBlue)
             {
-              image->colormap[i].blue=RoundToQuantum
+              image->colormap[i].blue=ClampToQuantum
                 (((double)image->colormap[i].blue*QuantumRange+(PalHeader.MaxBlue>>1))/PalHeader.MaxBlue);
             }
 
         }
+      if (EOFBlob(image))
+        ThrowCUTReaderException(CorruptImageError,"UnexpectedEndOfFile");
     }
 
 
@@ -505,13 +535,13 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
 
       image->colors=256;
-      if (AcquireImageColormap(image,image->colors) == MagickFalse)
+      if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
         {
         NoMemory:
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          ThrowCUTReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
 
-      for (i=0; i < (long)image->colors; i++)
+      for (i=0; i < (ssize_t)image->colors; i++)
         {
           image->colormap[i].red=ScaleCharToQuantum((unsigned char) i);
           image->colormap[i].green=ScaleCharToQuantum((unsigned char) i);
@@ -524,10 +554,18 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t) ldblk,
     sizeof(*BImgBuff));  /*Ldblk was set in the check phase*/
   if(BImgBuff==NULL) goto NoMemory;
+  (void) memset(BImgBuff,0,(size_t) ldblk*sizeof(*BImgBuff));
 
   offset=SeekBlob(image,6 /*sizeof(Header)*/,SEEK_SET);
   if (offset < 0)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    {
+      if (palette != NULL)
+        palette=DestroyImage(palette);
+      if (clone_info != NULL)
+        clone_info=DestroyImageInfo(clone_info);
+      BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
   for (i=0; i < (int) Header.Height; i++)
   {
       EncodedByte=ReadBlobLSBShort(image);
@@ -538,9 +576,9 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       RunCount=(unsigned char) ReadBlobByte(image);
       RunCountMasked=RunCount & 0x7F;
 
-      while((int) RunCountMasked!=0)
-        {
-          if((long) RunCountMasked>j)
+      while ((int) RunCountMasked != 0)
+      {
+          if((ssize_t) RunCountMasked>j)
             {    /*Wrong Data*/
               RunCountMasked=(unsigned char) j;
               if(j==0)
@@ -552,10 +590,10 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if((int) RunCount>0x80)
             {
               RunValue=(unsigned char) ReadBlobByte(image);
-              (void) ResetMagickMemory(ptrB,(int) RunValue,(size_t) RunCountMasked);
+              (void) memset(ptrB,(int) RunValue,(size_t) RunCountMasked);
             }
           else {
-            count=ReadBlob(image,(size_t) RunCountMasked,ptrB);
+            (void) ReadBlob(image,(size_t) RunCountMasked,ptrB);
           }
 
           ptrB+=(int) RunCountMasked;
@@ -566,22 +604,27 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           RunCountMasked=RunCount & 0x7F;
         }
 
-      InsertRow(depth,BImgBuff,i,image);
+      InsertRow(image,depth,BImgBuff,i,exception);
     }
+  (void) SyncImage(image,exception);
 
 
   /*detect monochrome image*/
 
   if(palette==NULL)
     {    /*attempt to detect binary (black&white) images*/
+      ImageType
+        type;
+
+      type=IdentifyImageCoderType(image,exception);
       if ((image->storage_class == PseudoClass) &&
-          (IsGrayImage(image,&image->exception) != MagickFalse))
+          (IsGrayImageType(type) != MagickFalse))
         {
-          if(GetCutColors(image)==2)
+          if(GetCutColors(image,exception)==2)
             {
-              for (i=0; i < (long)image->colors; i++)
+              for (i=0; i < (ssize_t)image->colors; i++)
                 {
-                  register Quantum
+                  Quantum
                     sample;
                   sample=ScaleCharToQuantum((unsigned char) i);
                   if(image->colormap[i].red!=sample) goto Finish;
@@ -589,19 +632,24 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   if(image->colormap[i].blue!=sample) goto Finish;
                 }
 
-              image->colormap[1].red=image->colormap[1].green=image->colormap[1].blue=(Quantum) QuantumRange;
-              for (i=0; i < (long)image->rows; i++)
+              image->colormap[1].red=image->colormap[1].green=
+                image->colormap[1].blue=QuantumRange;
+              for (i=0; i < (ssize_t)image->rows; i++)
                 {
-                  q=SetImagePixels(image,0,i,image->columns,1);
-                  for (j=0; j < (long)image->columns; j++)
+                  q=QueueAuthenticPixels(image,0,i,image->columns,1,exception);
+                  if (q == (Quantum *) NULL)
+                    break;
+                  for (j=0; j < (ssize_t)image->columns; j++)
                     {
-                      if(q->red==ScaleCharToQuantum(1))
+                      if (GetPixelRed(image,q) == ScaleCharToQuantum(1))
                         {
-                          q->red=q->green=q->blue=(Quantum) QuantumRange;
+                          SetPixelRed(image,QuantumRange,q);
+                          SetPixelGreen(image,QuantumRange,q);
+                          SetPixelBlue(image,QuantumRange,q);
                         }
-                      q++;
+                      q+=GetPixelChannels(image);
                     }
-                  if (SyncImagePixels(image) == MagickFalse) goto Finish;
+                  if (SyncAuthenticPixels(image,exception) == MagickFalse) goto Finish;
                 }
             }
         }
@@ -641,19 +689,17 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterCUTImage method is:
 %
-%      unsigned long RegisterCUTImage(void)
+%      size_t RegisterCUTImage(void)
 %
 */
-ModuleExport unsigned long RegisterCUTImage(void)
+ModuleExport size_t RegisterCUTImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("CUT");
+  entry=AcquireMagickInfo("CUT","CUT","DR Halo");
   entry->decoder=(DecodeImageHandler *) ReadCUTImage;
-  entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString("DR Halo");
-  entry->module=ConstantString("CUT");
+  entry->flags|=CoderDecoderSeekableStreamFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
