@@ -10,19 +10,21 @@
 %                            EEEEE  M   M  F                                  %
 %                                                                             %
 %                                                                             %
-%                  Read Windows Enahanced Metafile Format.                    %
+%                  Read Windows Enahanced Metafile Format                     %
 %                                                                             %
 %                              Software Design                                %
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
+%                               Dirk Lemstra                                  %
+%                               January 2014                                  %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -37,30 +39,39 @@
  * Include declarations.
  */
 
-#include "magick/studio.h"
+#include "MagickCore/studio.h"
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
-#  if defined(__CYGWIN__)
-#    include <windows.h>
+#  if !defined(_MSC_VER)
+#    if defined(__CYGWIN__)
+#      include <windows.h>
+#    else
+#      include <wingdi.h>
+#    endif
 #  else
-     /* All MinGW needs ... */
-#    include <wingdi.h>
+#pragma warning(disable: 4458)
+#    include <gdiplus.h>
+#pragma warning(default: 4458)
+#    pragma comment(lib, "gdiplus.lib")
 #  endif
 #endif
-
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/pixel.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "coders/emf.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -82,8 +93,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -117,8 +127,7 @@ static MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -159,19 +168,21 @@ static MagickBooleanType IsWMF(const unsigned char *magick,const size_t length)
 %
 %    o image_info: the image info..
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
-#if defined(MAGICKCORE_HAVE__WFOPEN)
+#if defined(MAGICKCORE_WINGDI32_DELEGATE)
+#  if !defined(_MSC_VER)
+#    if defined(MAGICKCORE_HAVE__WFOPEN)
 static size_t UTF8ToUTF16(const unsigned char *utf8,wchar_t *utf16)
 {
-  register const unsigned char
+  const unsigned char
     *p;
 
   if (utf16 != (wchar_t *) NULL)
     {
-      register wchar_t
+      wchar_t
         *q;
 
       wchar_t
@@ -258,17 +269,17 @@ static wchar_t *ConvertUTF8ToUTF16(const unsigned char *source)
   length=UTF8ToUTF16(source,(wchar_t *) NULL);
   if (length == 0)
     {
-      register long
+      ssize_t
         i;
 
       /*
         Not UTF-8, just copy.
       */
-      length=strlen(source);
+      length=strlen((char *) source);
       utf16=(wchar_t *) AcquireQuantumMemory(length+1,sizeof(*utf16));
       if (utf16 == (wchar_t *) NULL)
         return((wchar_t *) NULL);
-      for (i=0; i <= (long) length; i++)
+      for (i=0; i <= (ssize_t) length; i++)
         utf16[i]=source[i];
       return(utf16);
     }
@@ -278,19 +289,12 @@ static wchar_t *ConvertUTF8ToUTF16(const unsigned char *source)
   length=UTF8ToUTF16(source,utf16);
   return(utf16);
 }
-#endif
+#    endif /* MAGICKCORE_HAVE__WFOPEN */
 
-/*
-  This method reads either an enhanced metafile, a regular 16bit Windows
-  metafile, or an Aldus Placeable metafile and converts it into an enhanced
-  metafile.  Width and height are returned in .01mm units.
-*/
-#if defined(MAGICKCORE_WINGDI32_DELEGATE)
-static HENHMETAFILE ReadEnhMetaFile(const char *path,long *width,
-  long *height)
+static HENHMETAFILE ReadEnhMetaFile(const char *path,ssize_t *width,
+  ssize_t *height)
 {
-#pragma pack( push )
-#pragma pack( 2 )
+#pragma pack( push, 2 )
   typedef struct
   {
     DWORD dwKey;
@@ -335,7 +339,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *path,long *width,
       wchar_t
         *unicode_path;
 
-      unicode_path=ConvertUTF8ToUTF16(path);
+      unicode_path=ConvertUTF8ToUTF16((const unsigned char *) path);
       if (unicode_path != (wchar_t *) NULL)
         {
           hTemp=GetEnhMetaFileW(unicode_path);
@@ -403,9 +407,15 @@ static HENHMETAFILE ReadEnhMetaFile(const char *path,long *width,
     return(NULL);
   dwSize=GetFileSize(hFile,NULL);
   pBits=(LPBYTE) AcquireQuantumMemory(dwSize,sizeof(*pBits));
+  if (pBits == (LPBYTE) NULL)
+    {
+      CloseHandle(hFile);
+      return((HENHMETAFILE) NULL);
+    }
   ReadFile(hFile,pBits,dwSize,&dwSize,NULL);
   CloseHandle(hFile);
-  if (((PAPMHEADER) pBits)->dwKey != 0x9ac6cdd7l)
+  if ((((PAPMHEADER) pBits)->dwKey != 0x9ac6cdd7l) ||
+      (((PAPMHEADER) pBits)->wInch == 0))
     {
       pBits=(BYTE *) DestroyString((char *) pBits);
       return((HENHMETAFILE) NULL);
@@ -430,8 +440,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *path,long *width,
 
 #define CENTIMETERS_INCH 2.54
 
-static Image *ReadEMFImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   BITMAPINFO
     DIBinfo;
@@ -449,25 +458,28 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   Image
     *image;
 
-  long
-    height,
-    width,
-    y;
+  MagickBooleanType
+    status;
 
   RECT
     rect;
 
-  register long
+  ssize_t
     x;
 
-  register PixelPacket
+  Quantum
     *q;
 
   RGBQUAD
     *pBits,
     *ppBits;
 
-  image=AcquireImage(image_info);
+  ssize_t
+    height,
+    width,
+    y;
+
+  image=AcquireImage(image_info,exception);
   hemf=ReadEnhMetaFile(image_info->filename,&width,&height);
   if (hemf == (HENHMETAFILE) NULL)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -479,72 +491,71 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
 
       y_resolution=DefaultResolution;
       x_resolution=DefaultResolution;
-      if (image->y_resolution > 0)
+      if (image->resolution.y > 0)
         {
-          y_resolution=image->y_resolution;
+          y_resolution=image->resolution.y;
           if (image->units == PixelsPerCentimeterResolution)
             y_resolution*=CENTIMETERS_INCH;
         }
-      if (image->x_resolution > 0)
+      if (image->resolution.x > 0)
         {
-          x_resolution=image->x_resolution;
+          x_resolution=image->resolution.x;
           if (image->units == PixelsPerCentimeterResolution)
             x_resolution*=CENTIMETERS_INCH;
         }
-      image->rows=(unsigned long) ((height/1000.0/CENTIMETERS_INCH)*
-        y_resolution+0.5);
-      image->columns=(unsigned long) ((width/1000.0/CENTIMETERS_INCH)*
+      image->rows=(size_t) ((height/1000.0/CENTIMETERS_INCH)*y_resolution+0.5);
+      image->columns=(size_t) ((width/1000.0/CENTIMETERS_INCH)*
         x_resolution+0.5);
     }
   if (image_info->size != (char *) NULL)
     {
-      long
-        x;
-
       image->columns=width;
       image->rows=height;
-      x=0;
-      y=0;
-      (void) GetGeometry(image_info->size,&x,&y,&image->columns,&image->rows);
+      (void) GetGeometry(image_info->size,(ssize_t *) NULL,(ssize_t *) NULL,
+        &image->columns,&image->rows);
     }
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   if (image_info->page != (char *) NULL)
     {
       char
         *geometry;
 
-      long
-        sans;
-
-      register char
+      char
         *p;
 
       MagickStatusType
         flags;
 
+      ssize_t
+        sans;
+
       geometry=GetPageGeometry(image_info->page);
       p=strchr(geometry,'>');
       if (p == (char *) NULL)
         {
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
-          if (image->x_resolution != 0.0)
-            image->columns=(unsigned long) ((image->columns*
-              image->x_resolution)+0.5);
-          if (image->y_resolution != 0.0)
-            image->rows=(unsigned long) ((image->rows*image->y_resolution)+0.5);
+          if (image->resolution.x != 0.0)
+            image->columns=(size_t) floor((image->columns*image->resolution.x)+
+              0.5);
+          if (image->resolution.y != 0.0)
+            image->rows=(size_t) floor((image->rows*image->resolution.y)+0.5);
         }
       else
         {
           *p='\0';
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
-          if (image->x_resolution != 0.0)
-            image->columns=(unsigned long) (((image->columns*
-              image->x_resolution)/DefaultResolution)+0.5);
-          if (image->y_resolution != 0.0)
-            image->rows=(unsigned long) (((image->rows*image->y_resolution)/
+          if (image->resolution.x != 0.0)
+            image->columns=(size_t) floor(((image->columns*image->resolution.x)/
+              DefaultResolution)+0.5);
+          if (image->resolution.y != 0.0)
+            image->rows=(size_t) floor(((image->rows*image->resolution.y)/
               DefaultResolution)+0.5);
         }
+      (void) flags;
       geometry=DestroyString(geometry);
     }
   hDC=GetDC(NULL);
@@ -556,20 +567,15 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   /*
     Initialize the bitmap header info.
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
-  (void) ResetMagickMemory(&DIBinfo,0,sizeof(BITMAPINFO));
+  (void) memset(&DIBinfo,0,sizeof(BITMAPINFO));
   DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
-  DIBinfo.bmiHeader.biWidth=image->columns;
-  DIBinfo.bmiHeader.biHeight=(-1)*image->rows;
+  DIBinfo.bmiHeader.biWidth=(LONG) image->columns;
+  DIBinfo.bmiHeader.biHeight=(-1)*(LONG) image->rows;
   DIBinfo.bmiHeader.biPlanes=1;
   DIBinfo.bmiHeader.biBitCount=32;
   DIBinfo.bmiHeader.biCompression=BI_RGB;
-  hBitmap=CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,
-    NULL,0);
+  hBitmap=CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,NULL,
+    0);
   ReleaseDC(NULL,hDC);
   if (hBitmap == (HBITMAP) NULL)
     {
@@ -595,9 +601,9 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
     Initialize the bitmap to the image background color.
   */
   pBits=ppBits;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       pBits->rgbRed=ScaleQuantumToChar(image->background_color.red);
       pBits->rgbGreen=ScaleQuantumToChar(image->background_color.green);
@@ -607,28 +613,28 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   }
   rect.top=0;
   rect.left=0;
-  rect.right=image->columns;
-  rect.bottom=image->rows;
+  rect.right=(LONG) image->columns;
+  rect.bottom=(LONG) image->rows;
   /*
     Convert metafile pixels.
   */
   PlayEnhMetaFile(hDC,hemf,&rect);
   pBits=ppBits;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=SetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      q->red=ScaleCharToQuantum(pBits->rgbRed);
-      q->green=ScaleCharToQuantum(pBits->rgbGreen);
-      q->blue=ScaleCharToQuantum(pBits->rgbBlue);
-      q->opacity=OpaqueOpacity;
+      SetPixelRed(image,ScaleCharToQuantum(pBits->rgbRed),q);
+      SetPixelGreen(image,ScaleCharToQuantum(pBits->rgbGreen),q);
+      SetPixelBlue(image,ScaleCharToQuantum(pBits->rgbBlue),q);
+      SetPixelAlpha(image,OpaqueAlpha,q);
       pBits++;
-      q++;
+      q+=GetPixelChannels(image);
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   DeleteEnhMetaFile(hemf);
@@ -637,7 +643,184 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   DeleteObject(hBitmap);
   return(GetFirstImageInList(image));
 }
-#endif /* MAGICKCORE_WINGDI32_DELEGATE */
+#  else
+
+static inline void EMFSetDimensions(Image * image,Gdiplus::Image *source)
+{
+  if ((image->resolution.x <= 0.0) || (image->resolution.y <= 0.0))
+    return;
+
+  image->columns=(size_t) floor((Gdiplus::REAL) source->GetWidth()/
+    source->GetHorizontalResolution()*image->resolution.x+0.5);
+  image->rows=(size_t)floor((Gdiplus::REAL) source->GetHeight()/
+    source->GetVerticalResolution()*image->resolution.y+0.5);
+}
+
+static Image *ReadEMFImage(const ImageInfo *image_info,
+  ExceptionInfo *exception)
+{
+  Gdiplus::Bitmap
+    *bitmap;
+
+  Gdiplus::BitmapData
+     bitmap_data;
+
+  Gdiplus::GdiplusStartupInput
+    startup_input;
+
+  Gdiplus::Graphics
+    *graphics;
+
+  Gdiplus::Image
+    *source;
+
+  Gdiplus::Rect
+    rect;
+
+  GeometryInfo
+    geometry_info;
+
+  Image
+    *image;
+
+  MagickStatusType
+    flags;
+
+  Quantum
+    *q;
+
+  ssize_t
+    x;
+
+  ssize_t
+    y;
+
+  ULONG_PTR
+    token;
+
+  unsigned char
+    *p;
+
+  wchar_t
+    fileName[MagickPathExtent];
+
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickCoreSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+
+  image=AcquireImage(image_info,exception);
+  if (Gdiplus::GdiplusStartup(&token,&startup_input,NULL) != 
+    Gdiplus::Status::Ok)
+    ThrowReaderException(CoderError, "GdiplusStartupFailed");
+  MultiByteToWideChar(CP_UTF8,0,image->filename,-1,fileName,MagickPathExtent);
+  source=Gdiplus::Image::FromFile(fileName);
+  if (source == (Gdiplus::Image *) NULL)
+    {
+      Gdiplus::GdiplusShutdown(token);
+      ThrowReaderException(FileOpenError,"UnableToOpenFile");
+    }
+
+  image->resolution.x=source->GetHorizontalResolution();
+  image->resolution.y=source->GetVerticalResolution();
+  image->columns=(size_t) source->GetWidth();
+  image->rows=(size_t) source->GetHeight();
+  if (image_info->size != (char *) NULL)
+    {
+      (void) GetGeometry(image_info->size,(ssize_t *) NULL,(ssize_t *) NULL,
+        &image->columns,&image->rows);
+      image->resolution.x=source->GetHorizontalResolution()*image->columns/
+        source->GetWidth();
+      image->resolution.y=source->GetVerticalResolution()*image->rows/
+        source->GetHeight();
+      if (image->resolution.x == 0)
+        image->resolution.x=image->resolution.y;
+      else if (image->resolution.y == 0)
+        image->resolution.y=image->resolution.x;
+      else
+        image->resolution.x=image->resolution.y=MagickMin(
+          image->resolution.x,image->resolution.y);
+      EMFSetDimensions(image,source);
+    }
+  else if (image_info->density != (char *) NULL)
+    {
+      flags=ParseGeometry(image_info->density,&geometry_info);
+      image->resolution.x=geometry_info.rho;
+      image->resolution.y=geometry_info.sigma;
+      if ((flags & SigmaValue) == 0)
+        image->resolution.y=image->resolution.x;
+      EMFSetDimensions(image,source);
+    }
+  if (SetImageExtent(image,image->columns,image->rows,exception) == MagickFalse)
+    {
+      delete source;
+      Gdiplus::GdiplusShutdown(token);
+      return(DestroyImageList(image));
+    }
+  image->alpha_trait=BlendPixelTrait;
+  if (image->ping != MagickFalse)
+    {
+      delete source;
+      Gdiplus::GdiplusShutdown(token);
+      return(image);
+    }
+
+  bitmap=new Gdiplus::Bitmap((INT) image->columns,(INT) image->rows,
+    PixelFormat32bppARGB);
+  graphics=Gdiplus::Graphics::FromImage(bitmap);
+  graphics->SetInterpolationMode(Gdiplus::InterpolationModeHighQualityBicubic);
+  graphics->SetSmoothingMode(Gdiplus::SmoothingModeHighQuality);
+  graphics->SetTextRenderingHint(Gdiplus::TextRenderingHintClearTypeGridFit);
+  graphics->Clear(Gdiplus::Color((BYTE) ScaleQuantumToChar(
+    image->background_color.alpha),(BYTE) ScaleQuantumToChar(
+    image->background_color.red),(BYTE) ScaleQuantumToChar(
+    image->background_color.green),(BYTE) ScaleQuantumToChar(
+    image->background_color.blue)));
+  graphics->DrawImage(source,0,0,(INT) image->columns,(INT) image->rows);
+  delete graphics;
+  delete source;
+
+  rect=Gdiplus::Rect(0,0,(INT) image->columns,(INT) image->rows);
+  if (bitmap->LockBits(&rect,Gdiplus::ImageLockModeRead,PixelFormat32bppARGB,
+    &bitmap_data) != Gdiplus::Ok)
+  {
+    delete bitmap;
+    Gdiplus::GdiplusShutdown(token);
+    ThrowReaderException(FileOpenError,"UnableToReadImageData");
+  }
+
+  for (y=0; y < (ssize_t) image->rows; y++)
+  {
+    p=(unsigned char *) bitmap_data.Scan0+(y*abs(bitmap_data.Stride));
+    if (bitmap_data.Stride < 0)
+      q=GetAuthenticPixels(image,0,image->rows-y-1,image->columns,1,exception);
+    else
+      q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
+      break;
+
+    for (x=0; x < (ssize_t) image->columns; x++)
+    {
+      SetPixelBlue(image,ScaleCharToQuantum(*p++),q);
+      SetPixelGreen(image,ScaleCharToQuantum(*p++),q);
+      SetPixelRed(image,ScaleCharToQuantum(*p++),q);
+      SetPixelAlpha(image,ScaleCharToQuantum(*p++),q);
+      q+=GetPixelChannels(image);
+    }
+
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+  }
+
+  bitmap->UnlockBits(&bitmap_data);
+  delete bitmap;
+  Gdiplus::GdiplusShutdown(token);
+  return(image);
+}
+#  endif /* _MSC_VER */
+#endif /* MAGICKCORE_EMF_DELEGATE */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -659,32 +842,27 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterEMFImage method is:
 %
-%      unsigned long RegisterEMFImage(void)
+%      size_t RegisterEMFImage(void)
 %
 */
-ModuleExport unsigned long RegisterEMFImage(void)
+ModuleExport size_t RegisterEMFImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("EMF");
+  entry=AcquireMagickInfo("EMF","EMF","Windows Enhanced Meta File");
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
   entry->decoder=ReadEMFImage;
 #endif
-  entry->description=ConstantString(
-    "Windows WIN32 API rendered Enhanced Meta File");
   entry->magick=(IsImageFormatHandler *) IsEMF;
-  entry->blob_support=MagickFalse;
-  entry->module=ConstantString("WMF");
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("WMFWIN32");
+  entry=AcquireMagickInfo("EMF","WMF","Windows Meta File");
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
   entry->decoder=ReadEMFImage;
 #endif
-  entry->description=ConstantString("Windows WIN32 API rendered Meta File");
   entry->magick=(IsImageFormatHandler *) IsWMF;
-  entry->blob_support=MagickFalse;
-  entry->module=ConstantString("WMFWIN32");
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -711,5 +889,5 @@ ModuleExport unsigned long RegisterEMFImage(void)
 ModuleExport void UnregisterEMFImage(void)
 {
   (void) UnregisterMagickInfo("EMF");
-  (void) UnregisterMagickInfo("WMFWIN32");
+  (void) UnregisterMagickInfo("WMF");
 }
