@@ -17,7 +17,7 @@
 %                                 April 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,26 +39,26 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/resource_.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/utility.h"
 #if defined(MAGICKCORE_OPENEXR_DELEGATE)
 #include <ImfCRgbaFile.h>
 #if IMF_VERSION_NUMBER > 1
@@ -66,7 +66,7 @@
 #endif
 
 /*
-  Typedef declarations.
+  Typedef declaractions.
 */
 typedef struct _EXRWindowInfo
 {
@@ -81,7 +81,7 @@ typedef struct _EXRWindowInfo
   Forward declarations.
 */
 static MagickBooleanType
-  WriteEXRImage(const ImageInfo *,Image *);
+  WriteEXRImage(const ImageInfo *,Image *,ExceptionInfo *);
 #endif
 
 /*
@@ -163,10 +163,13 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ImfRgba
     *scanline;
 
+  int
+    compression;
+
   MagickBooleanType
     status;
 
-  PixelPacket
+  Quantum
     *q;
 
   size_t
@@ -180,12 +183,12 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickCoreSignature);
-  if (IsEventLogging() != MagickFalse)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  image=AcquireImage(image_info);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -203,28 +206,51 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
   hdr_info=ImfInputHeader(file);
   ImfHeaderDataWindow(hdr_info,&display_window.min_x,&display_window.min_y,
     &display_window.max_x,&display_window.max_y);
-  image->columns=(size_t) (display_window.max_x-display_window.min_x+1UL);
-  image->rows=(size_t) (display_window.max_y-display_window.min_y+1UL);
-  image->matte=MagickTrue;
-  (void) SetImageColorspace(image,RGBColorspace);
+  image->columns=((size_t) display_window.max_x-display_window.min_x+1UL);
+  image->rows=((size_t) display_window.max_y-display_window.min_y+1UL);
+  image->alpha_trait=BlendPixelTrait;
+  (void) SetImageColorspace(image,RGBColorspace,exception);
   image->gamma=1.0;
+  image->compression=NoCompression;
+  compression=ImfHeaderCompression(hdr_info);
+  if (compression == IMF_RLE_COMPRESSION)
+    image->compression=RLECompression;
+  if (compression == IMF_ZIPS_COMPRESSION)
+    image->compression=ZipSCompression;
+  if (compression == IMF_ZIP_COMPRESSION)
+    image->compression=ZipCompression;
+  if (compression == IMF_PIZ_COMPRESSION)
+    image->compression=PizCompression;
+  if (compression == IMF_PXR24_COMPRESSION)
+    image->compression=Pxr24Compression;
+#if defined(IMF_B44_COMPRESSION)
+  if (compression == IMF_B44_COMPRESSION)
+    image->compression=B44Compression;
+#endif
+#if defined(IMF_B44A_COMPRESSION)
+  if (compression == IMF_B44A_COMPRESSION)
+    image->compression=B44ACompression;
+#endif
+#if defined(IMF_DWAA_COMPRESSION)
+  if (compression == IMF_DWAA_COMPRESSION)
+    image->compression=DWAACompression;
+#endif
+#if defined(IMF_DWAB_COMPRESSION)
+  if (compression == IMF_DWAB_COMPRESSION)
+    image->compression=DWABCompression;
+#endif
   if (image_info->ping != MagickFalse)
     {
       (void) ImfCloseInputFile(file);
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      (void) ImfCloseInputFile(file);
-      (void) CloseBlob(image);
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+    return(DestroyImageList(image));
   ImfHeaderDataWindow(hdr_info,&data_window.min_x,&data_window.min_y,
     &data_window.max_x,&data_window.max_y);
-  columns=(size_t) (data_window.max_x-data_window.min_x+1UL);
+  columns=((size_t) data_window.max_x-data_window.min_x+1UL);
   if ((display_window.min_x > data_window.max_x) ||
       (display_window.min_x+(int) image->columns <= data_window.min_x))
     scanline=(ImfRgba *) NULL;
@@ -234,7 +260,6 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (scanline == (ImfRgba *) NULL)
         {
           (void) ImfCloseInputFile(file);
-          (void) CloseBlob(image);
           image=DestroyImageList(image);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
@@ -248,36 +273,25 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
       x;
 
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    if (q == (Quantum *) NULL)
       break;
     yy=(int) (display_window.min_y+y);
     if ((yy < data_window.min_y) || (yy > data_window.max_y) ||
         (scanline == (ImfRgba *) NULL))
-    {
-      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        SetPixelRed(q,image->background_color.red);
-        SetPixelGreen(q,image->background_color.green);
-        SetPixelBlue(q,image->background_color.blue);
-        SetPixelOpacity(q,image->background_color.opacity);
-        q++;
+        for (x=0; x < (ssize_t) image->columns; x++)
+        {
+          SetPixelViaPixelInfo(image,&image->background_color,q);
+          q+=GetPixelChannels(image);
+        }
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          break;
+        continue;
       }
-      if (SyncAuthenticPixels(image,exception) == MagickFalse)
-        break;
-      continue;
-    }
     (void) memset(scanline,0,columns*sizeof(*scanline));
-    if (ImfInputSetFrameBuffer(file,scanline-data_window.min_x-columns*yy,1,
-      columns) == 0)
-      {
-        status=MagickFalse;
-        break;
-      }
-    if (ImfInputReadPixels(file,yy,yy) == 0)
-      {
-        status=MagickFalse;
-        break;
-      }
+    ImfInputSetFrameBuffer(file,scanline-data_window.min_x-columns*yy,1,
+      columns);
+    ImfInputReadPixels(file,yy,yy);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
       int
@@ -285,36 +299,26 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       xx=(int) (display_window.min_x+x-data_window.min_x);
       if ((xx < 0) || (display_window.min_x+(int) x > data_window.max_x))
-        {
-          SetPixelRed(q,image->background_color.red);
-          SetPixelGreen(q,image->background_color.green);
-          SetPixelBlue(q,image->background_color.blue);
-          SetPixelOpacity(q,image->background_color.opacity);
-        }
+        SetPixelViaPixelInfo(image,&image->background_color,q);
       else
         {
-          SetPixelRed(q,ClampToQuantum((MagickRealType) QuantumRange*
-            (MagickRealType) ImfHalfToFloat(scanline[xx].r)));
-          SetPixelGreen(q,ClampToQuantum((MagickRealType) QuantumRange*
-            (MagickRealType) ImfHalfToFloat(scanline[xx].g)));
-          SetPixelBlue(q,ClampToQuantum((MagickRealType) QuantumRange*
-            (MagickRealType) ImfHalfToFloat(scanline[xx].b)));
-          SetPixelAlpha(q,ClampToQuantum((MagickRealType) QuantumRange*
-            (MagickRealType) ImfHalfToFloat(scanline[xx].a)));
+          SetPixelRed(image,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].r)),q);
+          SetPixelGreen(image,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].g)),q);
+          SetPixelBlue(image,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].b)),q);
+          SetPixelAlpha(image,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].a)),q);
         }
-      q++;
+      q+=GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   scanline=(ImfRgba *) RelinquishMagickMemory(scanline);
   (void) ImfCloseInputFile(file);
-  if (status == MagickFalse)
-    ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-  if (CloseBlob(image) == MagickFalse)
-    status=MagickFalse;
-  if (status == MagickFalse)
-    return(DestroyImageList(image));
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -351,7 +355,7 @@ ModuleExport size_t RegisterEXRImage(void)
     *entry;
 
   *version='\0';
-  entry=SetMagickInfo("EXR");
+  entry=AcquireMagickInfo("EXR","EXR","High Dynamic-range (HDR)");
 #if defined(MAGICKCORE_OPENEXR_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadEXRImage;
   entry->encoder=(EncodeImageHandler *) WriteEXRImage;
@@ -362,11 +366,9 @@ ModuleExport size_t RegisterEXRImage(void)
   entry->magick=(IsImageFormatHandler *) IsEXR;
   if (*version != '\0')
     entry->version=ConstantString(version);
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("High Dynamic-range (HDR)");
-  entry->blob_support=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->magick_module=ConstantString("EXR");
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -412,7 +414,8 @@ ModuleExport void UnregisterEXRImage(void)
 %
 %  The format of the WriteEXRImage method is:
 %
-%      MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteEXRImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -420,8 +423,11 @@ ModuleExport void UnregisterEXRImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   const char
     *sampling_factor,
@@ -450,7 +456,7 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  const PixelPacket
+  const Quantum
     *p;
 
   ssize_t
@@ -466,12 +472,14 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (IsEventLogging() != MagickFalse)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  (void) SetImageColorspace(image,RGBColorspace,exception);
   write_info=CloneImageInfo(image_info);
   (void) AcquireUniqueFilename(write_info->filename);
   hdr_info=ImfNewHeader();
@@ -480,6 +488,8 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   ImfHeaderSetDisplayWindow(hdr_info,0,0,(int) image->columns-1,(int)
     image->rows-1);
   compression=IMF_NO_COMPRESSION;
+  if (write_info->compression == RLECompression)
+    compression=IMF_RLE_COMPRESSION;
   if (write_info->compression == ZipSCompression)
     compression=IMF_ZIPS_COMPRESSION;
   if (write_info->compression == ZipCompression)
@@ -488,13 +498,21 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
     compression=IMF_PIZ_COMPRESSION;
   if (write_info->compression == Pxr24Compression)
     compression=IMF_PXR24_COMPRESSION;
-#if defined(B44Compression)
+#if defined(IMF_B44_COMPRESSION)
   if (write_info->compression == B44Compression)
     compression=IMF_B44_COMPRESSION;
 #endif
-#if defined(B44ACompression)
+#if defined(IMF_B44A_COMPRESSION)
   if (write_info->compression == B44ACompression)
     compression=IMF_B44A_COMPRESSION;
+#endif
+#if defined(IMF_DWAA_COMPRESSION)
+  if (write_info->compression == DWAACompression)
+    compression=IMF_DWAA_COMPRESSION;
+#endif
+#if defined(IMF_DWAB_COMPRESSION)
+  if (write_info->compression == DWABCompression)
+    compression=IMF_DWAB_COMPRESSION;
 #endif
   channels=0;
   value=GetImageOption(image_info,"exr:color-type");
@@ -521,8 +539,8 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
       else if (LocaleCompare(value,"A") == 0)
         channels=IMF_WRITE_A;
       else
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
-          CoderWarning,"ignoring invalid defined exr:color-type","=%s",value);
+        (void) ThrowMagickException(exception,GetMagickModule(),CoderWarning,
+          "ignoring invalid defined exr:color-type","=%s",value);
    }
   sampling_factor=(const char *) NULL;
   factors[0]=0;
@@ -548,8 +566,8 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
               factors[0]=0;
           }
       if ((factors[0] != 1) && (factors[0] != 2))
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
-          CoderWarning,"ignoring sampling-factor","=%s",sampling_factor);
+        (void) ThrowMagickException(exception,GetMagickModule(),CoderWarning,
+          "ignoring sampling-factor","=%s",sampling_factor);
       else if (channels != 0)
         {
           /*
@@ -558,7 +576,7 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
           factors[1]=((channels == IMF_WRITE_YCA) ||
             (channels == IMF_WRITE_YC)) ? 2 : 1;
           if (factors[0] != factors[1])
-            (void) ThrowMagickException(&image->exception,GetMagickModule(),
+            (void) ThrowMagickException(exception,GetMagickModule(),
               CoderWarning,"sampling-factor and color type mismatch","=%s",
               sampling_factor);
         }
@@ -569,9 +587,11 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
         If no color type given, select it now.
       */
       if (factors[0] == 2)
-        channels=image->matte ? IMF_WRITE_YCA : IMF_WRITE_YC;
+        channels=image->alpha_trait != UndefinedPixelTrait ? IMF_WRITE_YCA :
+          IMF_WRITE_YC;
       else
-        channels=image->matte ? IMF_WRITE_RGBA : IMF_WRITE_RGB;
+        channels=image->alpha_trait != UndefinedPixelTrait ? IMF_WRITE_RGBA :
+          IMF_WRITE_RGB;
     }
   ImfHeaderSetCompression(hdr_info,compression);
   ImfHeaderSetLineOrder(hdr_info,IMF_INCREASING_Y);
@@ -581,7 +601,7 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
     {
       (void) RelinquishUniqueFileResource(write_info->filename);
       write_info=DestroyImageInfo(write_info);
-      ThrowFileException(&image->exception,BlobError,"UnableToOpenBlob",
+      ThrowFileException(exception,BlobError,"UnableToOpenBlob",
         ImfErrorMessage());
       return(MagickFalse);
     }
@@ -596,38 +616,33 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   memset(scanline,0,image->columns*sizeof(*scanline));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      ImfFloatToHalf(QuantumScale*(MagickRealType) GetPixelRed(p),
-        &half_quantum);
+      ImfFloatToHalf(QuantumScale*GetPixelRed(image,p),&half_quantum);
       scanline[x].r=half_quantum;
-      ImfFloatToHalf(QuantumScale*(MagickRealType) GetPixelGreen(p),
-        &half_quantum);
+      ImfFloatToHalf(QuantumScale*GetPixelGreen(image,p),&half_quantum);
       scanline[x].g=half_quantum;
-      ImfFloatToHalf(QuantumScale*(MagickRealType) GetPixelBlue(p),
-        &half_quantum);
+      ImfFloatToHalf(QuantumScale*GetPixelBlue(image,p),&half_quantum);
       scanline[x].b=half_quantum;
-      if (image->matte == MagickFalse)
+      if (image->alpha_trait == UndefinedPixelTrait)
         ImfFloatToHalf(1.0,&half_quantum);
       else
-        ImfFloatToHalf(1.0-QuantumScale*(MagickRealType) GetPixelOpacity(p),
-          &half_quantum);
+        ImfFloatToHalf(QuantumScale*GetPixelAlpha(image,p),&half_quantum);
       scanline[x].a=half_quantum;
-      p++;
+      p+=GetPixelChannels(image);
     }
     ImfOutputSetFrameBuffer(file,scanline-(y*image->columns),1,image->columns);
     ImfOutputWritePixels(file,1);
   }
   (void) ImfCloseOutputFile(file);
   scanline=(ImfRgba *) RelinquishMagickMemory(scanline);
-  (void) FileToImage(image,write_info->filename);
+  (void) FileToImage(image,write_info->filename,exception);
   (void) RelinquishUniqueFileResource(write_info->filename);
   write_info=DestroyImageInfo(write_info);
-  if (CloseBlob(image) == MagickFalse)
-    status=MagickFalse;
-  return(status);
+  (void) CloseBlob(image);
+  return(MagickTrue);
 }
 #endif
