@@ -1,65 +1,61 @@
 /*
- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
- %                                                                             %
- %                                                                             %
- %                                                                             %
- %                     IIIIIIIIII    PPPPPPPP      LL                          %
- %                         II        PP      PP    LL                          %
- %                         II        PP       PP   LL                          %
- %                         II        PP      PP    LL                          %
- %                         II        PPPPPPPP      LL                          %
- %                         II        PP            LL                          %
- %                         II        PP            LL                          %
- %                     IIIIIIIIII    PP            LLLLLLLL                    %
- %                                                                             %
- %                                                                             %
- %                                                                             %
- %                   Read/Write Scanalytics IPLab Image Format                 %
- %                                  Sean Burke                                 %
- %                                  2008.05.07                                 %
- %                                     v 0.9                                   %
- %                                                                             %
- %  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
- %  dedicated to making software imaging solutions freely available.           %
- %                                                                             %
- %  You may not use this file except in compliance with the License.  You may  %
- %  obtain a copy of the License at                                            %
- %                                                                             %
- %    http://www.imagemagick.org/script/license.php                            %
- %                                                                             %
- %  Unless required by applicable law or agreed to in writing, software        %
- %  distributed under the License is distributed on an "AS IS" BASIS,          %
- %  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
- %  See the License for the specific language governing permissions and        %
- %  limitations under the License.                                             %
- %                                                                             %
- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
- %
- %
- */
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                             IIIII  PPPP   L                                 %
+%                               I    P   P  L                                 %
+%                               I    PPPP   L                                 %
+%                               I    P      L                                 %
+%                             IIIII  P      LLLLL                             %
+%                                                                             %
+%                 Read/Write Scanalytics IPLab Image Format                   %
+%                                Sean Burke                                   %
+%                                2008.05.07                                   %
+%                                   v 0.9                                     %
+%                                                                             %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
+%                                                                             %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
+%                                                                             %
+%    https://imagemagick.org/script/license.php                               %
+%                                                                             %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%
+*/
 
 /*
  Include declarations.
  */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/colorspace.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/option.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
 
 /* 
 Tyedef declarations
@@ -67,22 +63,23 @@ Tyedef declarations
 
 typedef struct _IPLInfo
 {
-  unsigned long
-  tag,
-  size,
-  time,
-  z,
-  width,
-  height,
-  colors,
-  depth,
-  byteType;
+  unsigned int
+    tag,
+    size,
+    time,
+    z,
+    width,
+    height,
+    colors,
+    depth,
+    byteType;
 } IPLInfo;
 
 static MagickBooleanType
-  WriteIPLImage(const ImageInfo *,Image *);
+  WriteIPLImage(const ImageInfo *,Image *,ExceptionInfo *);
 
-void increase (void *pixel, int byteType){
+/*
+static void increase (void *pixel, int byteType){
   switch(byteType){
     case 0:(*((unsigned char *) pixel))++; break;
     case 1:(*((signed int *) pixel))++; break;
@@ -91,7 +88,8 @@ void increase (void *pixel, int byteType){
     default:(*((unsigned int *) pixel))++; break;
   }  
 }
-
+*/
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -112,8 +110,7 @@ void increase (void *pixel, int byteType){
  %
  %  A description of each parameter follows:
  %
- %    o magick: This string is generally the first few bytes of an image file
- %      or blob.
+ %    o magick: compare image format pattern against these bytes.
  %
  %    o length: Specifies the length of the magick string.
  %
@@ -122,11 +119,11 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"data",4) == 0)
+  if (LocaleNCompare((const char *) magick,"data",4) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
-
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -160,12 +157,12 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
  %
  */
 
-void SetHeaderFromIPL(Image *image, IPLInfo *ipl){
-	image->columns = ipl->width;
-	image->rows = ipl->height;
-	image->depth = ipl->depth;
-	image->x_resolution = 1;
-	image->y_resolution = 1;
+static void SetHeaderFromIPL(Image *image, IPLInfo *ipl){
+  image->columns = ipl->width;
+  image->rows = ipl->height;
+  image->depth = ipl->depth;
+  image->resolution.x = 1;
+  image->resolution.y = 1;
 }
 
 
@@ -178,11 +175,11 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image *image;
 
   MagickBooleanType status;
-  register PixelPacket *q;
+  Quantum *q;
   unsigned char magick[12], *pixels;
   ssize_t count;
-  long y;
-  unsigned long t_count=0;
+  ssize_t y;
+  size_t t_count=0;
   size_t length;
   IPLInfo
     ipl_info;
@@ -193,18 +190,21 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   QuantumType
     quantum_type;
 
+  size_t
+    extent;
+
   /*
    Open Image
    */
 
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if ( image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent, GetMagickModule(), "%s",
                 image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
   {
@@ -220,7 +220,9 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Determine endianness 
    If we get back "iiii", we have LSB,"mmmm", MSB
    */
-  count=ReadBlob(image,4,magick); 
+  count=ReadBlob(image,4,magick);
+  if (count != 4)
+    ThrowReaderException(CorruptImageError, "ImproperImageHeader");
   if((LocaleNCompare((char *) magick,"iiii",4) == 0))  
     image->endian=LSBEndian;
   else{
@@ -236,7 +238,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
    Excellent, now we read the header unimpeded.
    */
   count=ReadBlob(image,4,magick); 
-  if((LocaleNCompare((char *) magick,"data",4) != 0))  
+  if((count != 4) || (LocaleNCompare((char *) magick,"data",4) != 0))
     ThrowReaderException(CorruptImageError, "ImproperImageHeader");
   ipl_info.size=ReadBlobLong(image); 
   ipl_info.width=ReadBlobLong(image); 
@@ -244,14 +246,17 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if((ipl_info.width == 0UL) || (ipl_info.height == 0UL))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   ipl_info.colors=ReadBlobLong(image); 
-  if(ipl_info.colors == 3){ image->colorspace=RGBColorspace;}
-  else { image->colorspace = GRAYColorspace; }
+  if (ipl_info.colors == 3)
+    {
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)      
+        SetImageColorspace(image,sRGBColorspace,exception);
+    }
+  else
+    image->colorspace = GRAYColorspace;
   ipl_info.z=ReadBlobLong(image); 
   ipl_info.time=ReadBlobLong(image); 
-
   ipl_info.byteType=ReadBlobLong(image); 
 
-
   /* Initialize Quantum Info */
 
   switch (ipl_info.byteType) {
@@ -291,43 +296,48 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       quantum_format = UnsignedQuantumFormat;
       break;
   }
+  extent=ipl_info.width*ipl_info.height*ipl_info.z*ipl_info.depth/8;
+  if (extent > GetBlobSize(image))
+    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
 
   /*
     Set number of scenes of image
   */
-
   SetHeaderFromIPL(image, &ipl_info);
 
   /* Thats all we need if we are pinging. */
   if (image_info->ping != MagickFalse)
-  {
-    (void) CloseBlob(image);
-    return(GetFirstImageInList(image));
-  }
+    {
+      (void) CloseBlob(image);
+      return(GetFirstImageInList(image));
+    }
   length=image->columns;
   quantum_type=GetQuantumType(image,exception);
  do
   {
     SetHeaderFromIPL(image, &ipl_info);
 
-  if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
+    if (status == MagickFalse)
       return(DestroyImageList(image));
-    }
 /*
-   printf("Length: %lu, Memory size: %lu\n", length,(unsigned long)(image->depth));
+   printf("Length: %.20g, Memory size: %.20g\n", (double) length,(double)
+     image->depth);
 */
      quantum_info=AcquireQuantumInfo(image_info,image);
-     (void) SetQuantumFormat(image,quantum_format,quantum_info);
-     pixels=GetQuantumPixels(quantum_info); 
+     if (quantum_info == (QuantumInfo *) NULL)
+       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+     status=SetQuantumFormat(image,quantum_info,quantum_format);
+     if (status == MagickFalse)
+       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+     pixels=(unsigned char *) GetQuantumPixels(quantum_info); 
      if(image->columns != ipl_info.width){
 /*
- 		printf("Columns not set correctly!  Wanted: %lu, got: %lu\n",
- 			ipl_info.width, image->columns);
+     printf("Columns not set correctly!  Wanted: %.20g, got: %.20g\n",
+       (double) ipl_info.width, (double) image->columns);
 */
      }
 
@@ -336,42 +346,46 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
      */
     
   if(ipl_info.colors == 1){
-      for(y = 0; y < (long) image->rows; y++){
+      for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+        if (q == (Quantum *) NULL)
                 break;
-	(void) ImportQuantumPixels(image,quantum_info,GrayQuantum,pixels);
-          if (SyncImagePixels(image) == MagickFalse)
-                  break;
-  	}
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+          GrayQuantum,pixels,exception);
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          break;
+    }
   }
   else{
-      for(y = 0; y < (long) image->rows; y++){
+      for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+        if (q == (Quantum *) NULL)
                 break;
-	(void) ImportQuantumPixels(image,quantum_info,RedQuantum,pixels);	
-        if (SyncImagePixels(image) == MagickFalse)
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+          RedQuantum,pixels,exception);  
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
       }
-      for(y = 0; y < (long) image->rows; y++){
+      for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-                break;
-	(void) ImportQuantumPixels(image,quantum_info,GreenQuantum,pixels);
-        if (SyncImagePixels(image) == MagickFalse)
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+        if (q == (Quantum *) NULL)
+          break;
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+          GreenQuantum,pixels,exception);
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
       }
-      for(y = 0; y < (long) image->rows; y++){
+      for(y = 0; y < (ssize_t) image->rows; y++){
         (void) ReadBlob(image, length*image->depth/8, pixels);
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-                break;
-	(void) ImportQuantumPixels(image,quantum_info,BlueQuantum,pixels);
-        if (SyncImagePixels(image) == MagickFalse)
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+        if (q == (Quantum *) NULL)
+          break;
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+          BlueQuantum,pixels,exception);
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
       }
    }
@@ -386,27 +400,30 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
                  image->filename);
       break;
     }
-   if(t_count < ipl_info.z * ipl_info.time){
+   if (t_count < (size_t) (ipl_info.z*ipl_info.time))
+     {
       /*
        Proceed to next image.
        */
-      AcquireNextImage(image_info, image);
+      AcquireNextImage(image_info,image,exception);
       if (GetNextImageInList(image) == (Image *) NULL)
-      {
-        image=DestroyImageList(image);
-        return((Image *) NULL);
-      }
+        {
+          status=MagickFalse;
+          break;
+        }
       image=SyncNextImageInList(image); 
       status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
         GetBlobSize(image));
       if (status == MagickFalse)
         break;
     }
-  } while (t_count < ipl_info.z*ipl_info.time);
+  } while (t_count < (size_t) (ipl_info.z*ipl_info.time));
   CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
-
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -423,23 +440,21 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
  %
  %
  */
-ModuleExport unsigned long RegisterIPLImage(void)
+ModuleExport size_t RegisterIPLImage(void)
 {
   MagickInfo
     *entry;
   
-  entry=SetMagickInfo("IPL");
+  entry=AcquireMagickInfo("IPL","IPL","IPL Image Sequence");
   entry->decoder=(DecodeImageHandler *) ReadIPLImage;
   entry->encoder=(EncodeImageHandler *) WriteIPLImage;
   entry->magick=(IsImageFormatHandler *) IsIPL;
-  entry->adjoin=MagickTrue;
-  entry->description=ConstantString("IPL Image Sequence");
-  entry->module=ConstantString("IPL");
-  entry->endian_support=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEndianSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
-
+
 /*
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                                                                             %
@@ -465,72 +480,81 @@ ModuleExport void UnregisterIPLImage(void)
 }
 
 /*
- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
- %                                                                             %
- %                                                                             %
- %                                                                             %
- %   W r i t e I P L I m a g e                                                 %
- %                                                                             %
- %                                                                             %
- %                                                                             %
- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
- %
- %  WriteIPLImage() writes an image to a file in Scanalytics IPLabimage format.
- %
- %  The format of the WriteIPLImage method is:
- %
- %      MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
- %
- %  A description of each parameter follows.
- %
- %    o image_info: The image info.
- %
- %    o image:  The image.
- %
- */
-
-static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   W r i t e I P L I m a g e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  WriteIPLImage() writes an image to a file in Scanalytics IPLabimage format.
+%
+%  The format of the WriteIPLImage method is:
+%
+%      MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
+%       Image *image,ExceptionInfo *exception)
+%
+%  A description of each parameter follows.
+%
+%    o image_info: The image info.
+%
+%    o image:  The image.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
+  IPLInfo
+    ipl_info;
+
   MagickBooleanType
     status;
   
   MagickOffsetType
     scene;
   
-  register const PixelPacket
+  const Quantum
     *p;
 
-  unsigned char
-  *pixels;
- 
-  long
-    y;
-  
-  IPLInfo
-    ipl_info;
-
   QuantumInfo
     *quantum_info;
 
+  size_t
+    imageListLength;
+
+  ssize_t
+    y;
+  
+  unsigned char
+    *pixels;
+ 
    /*
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   scene=0;
   
-
-  quantum_info=AcquireQuantumInfo(image_info, image);
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if ((quantum_info->format == UndefinedQuantumFormat) &&
-      (IsHighDynamicRangeImage(image,&image->exception) != MagickFalse))
-    (void) SetQuantumFormat(image,FloatingPointQuantumFormat,quantum_info);
+      (IsHighDynamicRangeImage(image,exception) != MagickFalse))
+    SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
   switch(quantum_info->depth){
   case 8: 
     ipl_info.byteType = 0;
@@ -559,20 +583,19 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
     break;
     
   }
-  ipl_info.z = GetImageListLength(image);
+  imageListLength=GetImageListLength(image);
+  ipl_info.z = (unsigned int) imageListLength;
   /* There is no current method for detecting whether we have T or Z stacks */
   ipl_info.time = 1;
-  ipl_info.width = image->columns;
-  ipl_info.height = image->rows;
-  
-  if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
-  
-  if(image->colorspace == RGBColorspace) { ipl_info.colors = 3; }
+  ipl_info.width = (unsigned int) image->columns;
+  ipl_info.height = (unsigned int) image->rows;
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace,exception);
+  if(IssRGBCompatibleColorspace(image->colorspace) != MagickFalse) { ipl_info.colors = 3; }
   else{ ipl_info.colors = 1; }
   
-  ipl_info.size = 28 + 
-    ((image->depth)/8)*ipl_info.height*ipl_info.width*ipl_info.colors*ipl_info.z;
+  ipl_info.size = (unsigned int) (28 + 
+    ((image->depth)/8)*ipl_info.height*ipl_info.width*ipl_info.colors*ipl_info.z);
   
   /* Ok!  Calculations are done.  Lets write this puppy down! */
   
@@ -584,14 +607,14 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   the code in here, but it is all but useless if you want to use IPLab. */
 
   if(image_info->endian == MSBEndian)
-    (void) WriteBlob(image, 4, (unsigned char *) "mmmm");
+    (void) WriteBlob(image, 4, (const unsigned char *) "mmmm");
   else{
     image->endian = LSBEndian;
-    (void) WriteBlob(image, 4, (unsigned char *) "iiii");
+    (void) WriteBlob(image, 4, (const unsigned char *) "iiii");
   }
   (void) WriteBlobLong(image, 4);
-  (void) WriteBlob(image, 4, (unsigned char *) "100f");
-  (void) WriteBlob(image, 4, (unsigned char *) "data");
+  (void) WriteBlob(image, 4, (const unsigned char *) "100f");
+  (void) WriteBlob(image, 4, (const unsigned char *) "data");
   (void) WriteBlobLong(image, ipl_info.size);
   (void) WriteBlobLong(image, ipl_info.width); 
   (void) WriteBlobLong(image, ipl_info.height);
@@ -603,73 +626,73 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobLong(image, ipl_info.time);
   (void) WriteBlobLong(image, ipl_info.byteType);
   
-
-  
- do
+  do
     {
       /*
   Convert MIFF to IPL raster pixels.
       */
-      pixels=GetQuantumPixels(quantum_info);
+      pixels=(unsigned char *) GetQuantumPixels(quantum_info);
   if(ipl_info.colors == 1){
   /* Red frame */
-  for(y = 0; y < (long) ipl_info.height; y++){
-    p=GetImagePixels(image,0,y,image->columns,1);
-    if (p == (PixelPacket *) NULL)
+  for(y = 0; y < (ssize_t) ipl_info.height; y++){
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-      (void) ExportQuantumPixels(image, quantum_info, GrayQuantum, pixels);
-      (void) WriteBlob(image, image->columns*image->depth/8, pixels);
+    (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+      GrayQuantum, pixels,exception);
+    (void) WriteBlob(image, image->columns*image->depth/8, pixels);
   }
 
 }
   if(ipl_info.colors == 3){
   /* Red frame */
-  for(y = 0; y < (long) ipl_info.height; y++){
-    p=GetImagePixels(image,0,y,image->columns,1);
-    if (p == (PixelPacket *) NULL)
+  for(y = 0; y < (ssize_t) ipl_info.height; y++){
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-      (void) ExportQuantumPixels(image, quantum_info, RedQuantum, pixels);
-      (void) WriteBlob(image, image->columns*image->depth/8, pixels);
+    (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+      RedQuantum, pixels,exception);
+    (void) WriteBlob(image, image->columns*image->depth/8, pixels);
   }
 
     /* Green frame */
-    for(y = 0; y < (long) ipl_info.height; y++){
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (PixelPacket *) NULL)
+    for(y = 0; y < (ssize_t) ipl_info.height; y++){
+      p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+      if (p == (const Quantum *) NULL)
         break;
-        (void) ExportQuantumPixels(image, quantum_info, GreenQuantum, pixels);
-        (void) WriteBlob(image, image->columns*image->depth/8, pixels);
+      (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+        GreenQuantum, pixels,exception);
+      (void) WriteBlob(image, image->columns*image->depth/8, pixels);
     }
     /* Blue frame */
-    for(y = 0; y < (long) ipl_info.height; y++){
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (PixelPacket *) NULL)
+    for(y = 0; y < (ssize_t) ipl_info.height; y++){
+      p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+      if (p == (const Quantum *) NULL)
         break;
-      (void) ExportQuantumPixels(image, quantum_info, BlueQuantum, pixels);
+      (void) ExportQuantumPixels(image,(CacheView *) NULL, quantum_info,
+        BlueQuantum, pixels,exception);
       (void) WriteBlob(image, image->columns*image->depth/8, pixels);
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+                image->rows);
           if (status == MagickFalse)
             break;
         }
     }
   }
-  quantum_info=DestroyQuantumInfo(quantum_info);
-      if (GetNextImageInList(image) == (Image *) NULL)
-  break;
+  if (GetNextImageInList(image) == (Image *) NULL)
+    break;
       image=SyncNextImageInList(image);
-      status=SetImageProgress(image,SaveImagesTag,scene++,
-        GetImageListLength(image));
+      status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
       if (status == MagickFalse)
         break;
     }while (image_info->adjoin != MagickFalse);
 
-  (void) WriteBlob(image, 4, (unsigned char *) "fini");
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  (void) WriteBlob(image, 4, (const unsigned char *) "fini");
   (void) WriteBlobLong(image, 0);
 
 CloseBlob(image);
 return(MagickTrue);
 }
-
-
