@@ -13,17 +13,17 @@
 %                      Read ASCII String As An Image.                         %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,24 +39,26 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/annotate.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/draw.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/annotate.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -84,16 +86,28 @@
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline void AdjustTypeMetricBounds(TypeMetric *metrics)
+{
+  double
+    x1;
+      
+  if (metrics->bounds.x1 >= 0.0)
+    {
+      metrics->bounds.x1=0.0;
+      return;
+    }
+  x1=ceil(-metrics->bounds.x1+0.5);
+  metrics->width+=x1+x1;
+  metrics->bounds.x1=x1;
+}
+
 static Image *ReadLABELImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
-    geometry[MaxTextExtent],
-    *property;
-
-  const char
+    geometry[MagickPathExtent],
     *label;
 
   DrawInfo
@@ -108,7 +122,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
   TypeMetric
     metrics;
 
-  unsigned long
+  size_t
     height,
     width;
 
@@ -116,82 +130,169 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
     Initialize Image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   (void) ResetImagePage(image,"0x0+0+0");
-  property=InterpretImageProperties(image_info,image,image_info->filename);
-  (void) SetImageProperty(image,"label",property);
-  property=DestroyString(property);
-  label=GetImageProperty(image,"label");
-  draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->text=ConstantString(label);
-  if ((image->columns != 0) || (image->rows != 0))
+  if ((image->columns != 0) && (image->rows != 0))
     {
-      /*
-        Fit label to canvas size.
-      */
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-      for ( ; status != MagickFalse; draw_info->pointsize*=2.0)
-      {
-        width=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
-        height=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
-        if (((image->columns != 0) && (width > (image->columns+1))) ||
-            ((image->rows != 0) && (height > (image->rows+1))))
-          break;
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-      }
-      for ( ; status != MagickFalse; draw_info->pointsize--)
-      {
-        width=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
-        height=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
-        if ((image->columns != 0) && (width <= (image->columns+1)) &&
-           ((image->rows == 0) || (height <= (image->rows+1))))
-          break;
-        if ((image->rows != 0) && (height <= (image->rows+1)) &&
-           ((image->columns == 0) || (width <= (image->columns+1))))
-          break;
-        if (draw_info->pointsize < 2.0)
-          break;
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-      }
+      status=SetImageExtent(image,image->columns,image->rows,exception);
+      if (status == MagickFalse)
+        return(DestroyImageList(image));
+      (void) SetImageBackgroundColor(image,exception);
     }
-  status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+  label=InterpretImageProperties((ImageInfo *) image_info,image,
+    image_info->filename,exception);
+  if (label == (char *) NULL)
+    return(DestroyImageList(image));
+  (void) SetImageProperty(image,"label",label,exception);
+  draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+  width=(size_t) floor(0.5*draw_info->pointsize*strlen(label)+0.5);
+  if (AcquireMagickResource(WidthResource,width) == MagickFalse)
+    {
+      label=DestroyString(label);
+      draw_info=DestroyDrawInfo(draw_info);
+      ThrowReaderException(ImageError,"WidthOrHeightExceedsLimit");
+    }
+  draw_info->text=ConstantString(label);
+  (void) memset(&metrics,0,sizeof(metrics));
+  status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+  AdjustTypeMetricBounds(&metrics);
+  if ((image->columns == 0) && (image->rows == 0))
+    {
+      image->columns=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+      image->rows=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
+    }
+  else
+    if ((status != MagickFalse) && (strlen(label) > 0) &&
+        (((image->columns == 0) || (image->rows == 0)) ||
+         (fabs(image_info->pointsize) < MagickEpsilon)))
+      {
+        double
+          high,
+          low;
+
+        ssize_t
+          n;
+
+        /*
+          Auto fit text into bounding box.
+        */
+        for (n=0; n < 32; n++, draw_info->pointsize*=2.0)
+        {
+          (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+            metrics.bounds.x1,metrics.ascent);
+          if (draw_info->gravity == UndefinedGravity)
+            (void) CloneString(&draw_info->geometry,geometry);
+          status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+          if (status == MagickFalse)
+            break;
+          AdjustTypeMetricBounds(&metrics);
+          width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+          height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
+          if ((image->columns != 0) && (image->rows != 0))
+            {
+              if ((width >= image->columns) && (height >= image->rows))
+                break;
+            }
+          else
+            if (((image->columns != 0) && (width >= image->columns)) ||
+                ((image->rows != 0) && (height >= image->rows)))
+              break;
+        }
+        if (status == MagickFalse)
+          {
+            label=DestroyString(label);
+            draw_info=DestroyDrawInfo(draw_info);
+            image=DestroyImageList(image);
+            return((Image *) NULL);
+          }
+        high=draw_info->pointsize;
+        for (low=1.0; (high-low) > 0.5; )
+        {
+          draw_info->pointsize=(low+high)/2.0;
+          (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+            metrics.bounds.x1,metrics.ascent);
+          if (draw_info->gravity == UndefinedGravity)
+            (void) CloneString(&draw_info->geometry,geometry);
+          status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+          if (status == MagickFalse)
+            break;
+          AdjustTypeMetricBounds(&metrics);
+          width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+          height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
+          if ((image->columns != 0) && (image->rows != 0))
+            {
+              if ((width < image->columns) && (height < image->rows))
+                low=draw_info->pointsize+0.5;
+              else
+                high=draw_info->pointsize-0.5;
+            }
+          else
+            if (((image->columns != 0) && (width < image->columns)) ||
+                ((image->rows != 0) && (height < image->rows)))
+              low=draw_info->pointsize+0.5;
+            else
+              high=draw_info->pointsize-0.5;
+        }
+        if (status != MagickFalse)
+          {
+            draw_info->pointsize=floor((low+high)/2.0-0.5);
+            status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+            AdjustTypeMetricBounds(&metrics);
+          }
+      }
+   label=DestroyString(label);
+   if (status == MagickFalse)
+     {
+       draw_info=DestroyDrawInfo(draw_info);
+       image=DestroyImageList(image);
+       return((Image *) NULL);
+     }
+  if (image->columns == 0)
+    image->columns=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+  if (image->columns == 0)
+    image->columns=(size_t) floor(draw_info->pointsize+draw_info->stroke_width+
+      0.5);
+  if (image->rows == 0)
+    image->rows=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
+  if (image->rows == 0)
+    image->rows=(size_t) floor(draw_info->pointsize+draw_info->stroke_width+
+      0.5);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     {
-      InheritException(exception,&image->exception);
+      draw_info=DestroyDrawInfo(draw_info);
+      return(DestroyImageList(image));
+    }
+  if (SetImageBackgroundColor(image,exception) == MagickFalse)
+    {
+      draw_info=DestroyDrawInfo(draw_info);
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  if (image->columns == 0)
-    image->columns=(unsigned long) (metrics.width+draw_info->stroke_width+1.5);
-  if (image->columns == 0)
-    image->columns=(unsigned long) (draw_info->pointsize+
-      draw_info->stroke_width+1.5);
-  if (draw_info->gravity == UndefinedGravity)
-    {
-      (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
-        -metrics.bounds.x1+draw_info->stroke_width/2.0,metrics.ascent+
-        draw_info->stroke_width/2.0);
-      draw_info->geometry=AcquireString(geometry);
-    }
-  if (image->rows == 0)
-    image->rows=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
-  if (image->rows == 0)
-    image->rows=(unsigned long) (draw_info->pointsize+draw_info->stroke_width+
-      0.5);
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
-  (void) SetImageBackgroundColor(image);
-  (void) AnnotateImage(image,draw_info);
+  /*
+    Draw label.
+  */
+  (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+    (draw_info->direction == RightToLeftDirection ? (double) image->columns-
+    metrics.bounds.x2 : metrics.bounds.x1),(draw_info->gravity ==
+    UndefinedGravity ? MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0));
+  (void) CloneString(&draw_info->geometry,geometry);
+  status=AnnotateImage(image,draw_info,exception);
+  if (image_info->pointsize == 0.0)
+    (void) FormatImageProperty(image,"label:pointsize","%.20g",
+      draw_info->pointsize);
   draw_info=DestroyDrawInfo(draw_info);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   return(GetFirstImageInList(image));
 }
 
@@ -215,20 +316,18 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterLABELImage method is:
 %
-%      unsigned long RegisterLABELImage(void)
+%      size_t RegisterLABELImage(void)
 %
 */
-ModuleExport unsigned long RegisterLABELImage(void)
+ModuleExport size_t RegisterLABELImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("LABEL");
+  entry=AcquireMagickInfo("LABEL","LABEL","Image label");
   entry->decoder=(DecodeImageHandler *) ReadLABELImage;
-  entry->adjoin=MagickFalse;
+  entry->flags^=CoderAdjoinFlag;
   entry->format_type=ImplicitFormatType;
-  entry->description=ConstantString("Image label");
-  entry->module=ConstantString("LABEL");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
