@@ -13,17 +13,17 @@
 %                          Read a Plasma Image.                               %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,25 +39,31 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/constitute.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/random_.h"
-#include "magick/signature-private.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/channel.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/random_.h"
+#include "MagickCore/random-private.h"
+#include "MagickCore/signature-private.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/visual-effects.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -86,32 +92,24 @@
 %
 */
 
-static inline size_t MagickMax(const size_t x,const size_t y)
+static inline MagickBooleanType PlasmaPixel(Image *image,
+  RandomInfo *magick_restrict random_info,const double x,const double y,
+  ExceptionInfo *exception)
 {
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-static inline void PlasmaPixel(Image *image,double x,double y)
-{
-  QuantumAny
-    scale;
-
-  register PixelPacket
+  Quantum
     *q;
 
-  q=GetImagePixels(image,(long) ceil(x-0.5),(long) ceil(y-0.5),1,1);
-  if (q == (PixelPacket *) NULL)
-    return;
-  scale=GetQuantumScale(16UL);
-  q->red=ScaleAnyToQuantum((unsigned long) (65535.0*GetPseudoRandomValue()+
-    0.5),16UL,scale);
-  q->green=ScaleAnyToQuantum((unsigned long) (65535.0*GetPseudoRandomValue()+
-    0.5),16UL,scale);
-  q->blue=ScaleAnyToQuantum((unsigned long) (65535.0*GetPseudoRandomValue()+
-    0.5),16UL,scale);
-  (void) SyncImagePixels(image);
+  q=GetAuthenticPixels(image,(ssize_t) (x+0.5),(ssize_t) (y+0.5),1,1,
+    exception);
+  if (q == (Quantum *) NULL)
+    return(MagickFalse);
+  SetPixelRed(image,(Quantum) (QuantumRange*
+    GetPseudoRandomValue(random_info)+0.5),q);
+  SetPixelGreen(image,(Quantum) (QuantumRange*
+    GetPseudoRandomValue(random_info)+0.5),q);
+  SetPixelBlue(image,(Quantum) (QuantumRange*
+    GetPseudoRandomValue(random_info)+0.5),q);
+  return(SyncAuthenticPixels(image,exception));
 }
 
 static Image *ReadPlasmaImage(const ImageInfo *image_info,
@@ -123,51 +121,53 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   ImageInfo
     *read_info;
 
-  long
-    y;
-
-  MagickBooleanType
+  MagickStatusType
     status;
 
-  register long
+  ssize_t
     x;
 
-  register PixelPacket
+  Quantum
     *q;
 
-  register unsigned long
+  size_t
     i;
 
   SegmentInfo
     segment_info;
 
-  unsigned long
+  size_t
     depth,
     max_depth;
 
+  ssize_t
+    y;
+
   /*
     Recursively apply plasma to the image.
   */
   read_info=CloneImageInfo(image_info);
   SetImageInfoBlob(read_info,(void *) NULL,0);
-  (void) FormatMagickString(read_info->filename,MaxTextExtent,
+  (void) FormatLocaleString(read_info->filename,MagickPathExtent,
     "gradient:%s",image_info->filename);
   image=ReadImage(read_info,exception);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     return((Image *) NULL);
-  image->storage_class=DirectClass;
-  for (y=0; y < (long) image->rows; y++)
+  (void) SetImageStorageClass(image,DirectClass,exception);
+  if (IsGrayColorspace(image->colorspace) != MagickFalse)
+    (void) SetImageColorspace(image,sRGBColorspace,exception); 
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      q->opacity=(Quantum) (QuantumRange/2);
-      q++;
+      SetPixelAlpha(image,QuantumRange/2,q);
+      q+=GetPixelChannels(image);
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   segment_info.x1=0;
@@ -176,32 +176,48 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   segment_info.y2=(double) image->rows-1;
   if (LocaleCompare(image_info->filename,"fractal") == 0)
     {
+      RandomInfo
+        *random_info;
+
       /*
         Seed pixels before recursion.
       */
-      PlasmaPixel(image,segment_info.x1,segment_info.y1);
-      PlasmaPixel(image,segment_info.x1,(segment_info.y1+segment_info.y2)/2);
-      PlasmaPixel(image,segment_info.x1,segment_info.y2);
-      PlasmaPixel(image,(segment_info.x1+segment_info.x2)/2,segment_info.y1);
-      PlasmaPixel(image,(segment_info.x1+segment_info.x2)/2,
-        (segment_info.y1+segment_info.y2)/2);
-      PlasmaPixel(image,(segment_info.x1+segment_info.x2)/2,segment_info.y2);
-      PlasmaPixel(image,segment_info.x2,segment_info.y1);
-      PlasmaPixel(image,segment_info.x2,(segment_info.y1+segment_info.y2)/2);
-      PlasmaPixel(image,segment_info.x2,segment_info.y2);
+      (void) SetImageColorspace(image,sRGBColorspace,exception);
+      random_info=AcquireRandomInfo();
+      status=PlasmaPixel(image,random_info,segment_info.x1,segment_info.y1,
+        exception);
+      status&=PlasmaPixel(image,random_info,segment_info.x1,(segment_info.y1+
+        segment_info.y2)/2,exception);
+      status&=PlasmaPixel(image,random_info,segment_info.x1,segment_info.y2,
+        exception);
+      status&=PlasmaPixel(image,random_info,(segment_info.x1+segment_info.x2)/2,
+        segment_info.y1,exception);
+      status&=PlasmaPixel(image,random_info,(segment_info.x1+segment_info.x2)/2,
+        (segment_info.y1+segment_info.y2)/2,exception);
+      status&=PlasmaPixel(image,random_info,(segment_info.x1+segment_info.x2)/2,
+        segment_info.y2,exception);
+      status&=PlasmaPixel(image,random_info,segment_info.x2,segment_info.y1,
+        exception);
+      status&=PlasmaPixel(image,random_info,segment_info.x2,(segment_info.y1+
+        segment_info.y2)/2,exception);
+      status&=PlasmaPixel(image,random_info,segment_info.x2,segment_info.y2,
+        exception);
+      random_info=DestroyRandomInfo(random_info);
+      if (status == MagickFalse)
+        return(image);  
     }
-  i=(unsigned long) MagickMax(image->columns,image->rows)/2;
+  i=(size_t) MagickMax(image->columns,image->rows)/2;
   for (max_depth=0; i != 0; max_depth++)
     i>>=1;
   for (depth=1; ; depth++)
   {
-    if (PlasmaImage(image,&segment_info,0,depth) != MagickFalse)
+    if (PlasmaImage(image,&segment_info,0,depth,exception) != MagickFalse)
       break;
-    status=SetImageProgress(image,LoadImageTag,depth,max_depth);
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) depth,
+      max_depth);
     if (status == MagickFalse)
       break;
   }
-  (void) SetImageAlphaChannel(image,DeactivateAlphaChannel);
   return(GetFirstImageInList(image));
 }
 
@@ -225,27 +241,23 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterPLASMAImage method is:
 %
-%      unsigned long RegisterPLASMAImage(void)
+%      size_t RegisterPLASMAImage(void)
 %
 */
-ModuleExport unsigned long RegisterPLASMAImage(void)
+ModuleExport size_t RegisterPLASMAImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("PLASMA");
+  entry=AcquireMagickInfo("PLASMA","PLASMA","Plasma fractal image");
   entry->decoder=(DecodeImageHandler *) ReadPlasmaImage;
-  entry->adjoin=MagickFalse;
+  entry->flags^=CoderAdjoinFlag;
   entry->format_type=ImplicitFormatType;
-  entry->description=ConstantString("Plasma fractal image");
-  entry->module=ConstantString("PLASMA");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("FRACTAL");
+  entry=AcquireMagickInfo("PLASMA","FRACTAL","Plasma fractal image");
   entry->decoder=(DecodeImageHandler *) ReadPlasmaImage;
-  entry->adjoin=MagickFalse;
+  entry->flags^=CoderAdjoinFlag;
   entry->format_type=ImplicitFormatType;
-  entry->description=ConstantString("Plasma fractal image");
-  entry->module=ConstantString("PLASMA");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
