@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +49,7 @@
 #include "MagickCore/channel.h"
 #include "MagickCore/color.h"
 #include "MagickCore/color-private.h"
+#include "MagickCore/colorspace-private.h"
 #include "MagickCore/compress.h"
 #include "MagickCore/constitute.h"
 #include "MagickCore/draw.h"
@@ -74,6 +75,7 @@
 #include "MagickCore/timer-private.h"
 #include "MagickCore/token.h"
 #include "MagickCore/utility.h"
+#include "coders/coders-private.h"
 
 /*
   Define declarations.
@@ -244,13 +246,13 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -267,6 +269,7 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   if (*pixel_info == (MemoryInfo *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   q=(unsigned char *) GetVirtualMemoryBlob(*pixel_info);
+  (void) memset(q,0,*length*sizeof(*q));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -308,13 +311,13 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -385,13 +388,13 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -414,7 +417,7 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      *q++=(unsigned char) GetPixelIndex(image,p);
+      *q++=(unsigned char) ((ssize_t) GetPixelIndex(image,p));
       p+=GetPixelChannels(image);
     }
     if (image->previous == (Image *) NULL)
@@ -450,7 +453,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   MemoryInfo
     *pixel_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -812,7 +815,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
 
   char
     buffer[MagickPathExtent],
-    date[MagickPathExtent],
+    date[MagickTimeExtent],
     **labels,
     page_geometry[MagickPathExtent];
 
@@ -854,7 +857,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
     media_info,
     page_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   SegmentInfo
@@ -932,6 +935,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
   imageListLength=GetImageListLength(image);
   do
   {
+    ImageType
+      type;
+
     /*
       Scale relative to dots-per-inch.
     */
@@ -997,6 +1003,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
     if (value != (const char *) NULL)
       text_size=(size_t) (MultilineCensus(value)*pointsize+12);
     page++;
+    type=IdentifyImageCoderType(image,exception);
     if (page == 1)
       {
         /*
@@ -1015,7 +1022,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
           image->filename);
         (void) WriteBlobString(image,buffer);
         timer=GetMagickTime();
-        (void) FormatMagickTime(timer,MagickPathExtent,date);
+        (void) FormatMagickTime(timer,sizeof(date),date);
         (void) FormatLocaleString(buffer,MagickPathExtent,
           "%%%%CreationDate: %s\n",date);
         (void) WriteBlobString(image,buffer);
@@ -1043,7 +1050,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
               (void) WriteBlobString(image,
                 "%%DocumentProcessColors: Cyan Magenta Yellow Black\n");
             else
-              if (SetImageGray(image,exception) != MagickFalse)
+              if (IsGrayImageType(type) != MagickFalse)
                 (void) WriteBlobString(image,
                   "%%DocumentProcessColors: Black\n");
           }
@@ -1121,7 +1128,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
       (void) WriteBlobString(image,
         "%%PageProcessColors: Cyan Magenta Yellow Black\n");
     else
-      if (SetImageGray(image,exception) != MagickFalse)
+      if (IsGrayImageType(type) != MagickFalse)
         (void) WriteBlobString(image,"%%PageProcessColors: Black\n");
     /*
       Adjust document bounding box to bound page bounding box.
@@ -1148,9 +1155,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
       (void) WriteBlobString(image,"/ClipImage {} def\n");
     else
       {
-        const char
-          *value;
-
         value=GetImageProperty(image,image->magick_filename,exception);
         if (value == (const char *) NULL)
           return(MagickFalse);
@@ -1262,8 +1266,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image,
         (image_info->type != ColorSeparationType) &&
         (image_info->type != ColorSeparationAlphaType) &&
         (image->colorspace != CMYKColorspace) &&
-        ((SetImageGray(image,exception) != MagickFalse) ||
-         (SetImageMonochrome(image,exception) != MagickFalse)))
+        (IsGrayImageType(type) != MagickFalse))
       {
         /*
           Gray images.
